from os import environ

from AvenirCommon.Database import GB_get_db_json

import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.IS.ISConst as isc
import SpectrumCommon.Const.IS.ISDatabaseKeys as isdbk
import SpectrumCommon.Modvars.IS.ISModVarUtil as ismvu
import SpectrumCommon.Modvars.IS.ISModVarDefs as ismvd
import SpectrumCommon.Const.IS.ISConstLink as iscl
import SpectrumCommon.Const.IS.ISModVarFields as ismvf

#######################################################################################################
#                                                                                                     #
#                                       Database loading methods                                      #
#                                                                                                     #
#######################################################################################################

def IS_load_facility_med_equip_group_DB():
    facility_med_equip_group_DB_file_name = isc.IS_FACILITY_MED_EQUIP_GROUP_DB_NAME + '_' + isc.IS_FACILITY_EQUIP_DB_CURR_VERSION + '.' + gbc.GB_JSON
    facility_med_equip_group_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IS_CONTAINER, facility_med_equip_group_DB_file_name)

    return facility_med_equip_group_DB

def IS_load_facility_equip_input_DB():
    facility_equip_input_DB_file_name = isc.IS_FACILITY_EQUIP_DB_NAME + '_' + isc.IS_FACILITY_EQUIP_DB_CURR_VERSION + '.' + gbc.GB_JSON
    facility_equip_input_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_IS_CONTAINER, facility_equip_input_DB_file_name)

    return facility_equip_input_DB

#######################################################################################################
#                                                                                                     #
#                               Facility medical equipment group (MEG) records                        #
#                                                                                                     #
#######################################################################################################

def IS_init_facility_MEG_records(facility_med_equip_group_DB, TB_costing_on):
    facility_MEG_records = []

    for facility_med_equip_group_DB_obj in facility_med_equip_group_DB:
        group_mst_ID = facility_med_equip_group_DB_obj[isdbk.IS_GROUP_MST_ID_KEY_FEIDB]
    
        # Medical equipment groups are only initialized for the default medical equipment group ID.
        if (
            (TB_costing_on and (facility_med_equip_group_DB_obj[isdbk.IS_MOD_ID_KEY_FEIDB] == gbc.GB_TB))
            or ((not TB_costing_on) and (facility_med_equip_group_DB_obj[isdbk.IS_MOD_ID_KEY_FEIDB] != gbc.GB_TB))
            and group_mst_ID == isc.IS_DEF_MED_EQUIP_GROUP_ID
        ):
            facility_type_mst_ID = facility_med_equip_group_DB_obj[isdbk.IS_FACILITY_TYPE_MST_ID_KEY_FEIDB]
            group_name = facility_med_equip_group_DB_obj[isdbk.IS_GROUP_NAME_ID_KEY_FEIDB]
            group_string_constant = facility_med_equip_group_DB_obj[isdbk.IS_GROUP_STR_CONST_KEY_FEIDB]
            facility_MEG_record = ismvd.IS_create_facility_MEG_record(group_mst_ID, group_name, facility_type_mst_ID)
            ismvu.set_IS_facility_MEG_string_constant(facility_MEG_record, group_string_constant)
            ismvu.add_IS_facility_MEG_rec(facility_MEG_records, facility_MEG_record)

    # Normally, default groups are determined by looping over the equipment database and finding all the groups used for the various equipment. 
    # As a result, if we want empty groups, we manually add them here.
    if TB_costing_on:
        # Add a blank medical equipment group for Prehospital emergency.
        facility_type_mst_ID = str(isc.IS_PREHOSPITAL_EMERGENCY_MST_ID)
        group_mst_ID = '1'
        group_name = 'TB equipment'
        group_string_constant = 'GB_stTBEquipment'
        facility_MEG_record = ismvd.IS_create_facility_MEG_record(group_mst_ID, group_name, facility_type_mst_ID)
        ismvu.set_IS_facility_MEG_string_constant(facility_MEG_record, group_string_constant)
        ismvu.add_IS_facility_MEG_rec(facility_MEG_records, facility_MEG_record)

    return facility_MEG_records

#######################################################################################################
#                                                                                                     #
#                                       Facility equipment records                                    #
#                                                                                                     #
#######################################################################################################

# 
def IS_init_facility_equip_records(facility_equip_input_DB, costing_mode, IHT_plan_type, facility_type_ID : str = str(gbc.GB_NOT_FOUND)):

    facility_med_equip_dict_list, facility_furniture_dict_list = [], []

    for facility_equip_input_DB_obj in facility_equip_input_DB:
        facility_MEG_mst_ID = facility_equip_input_DB_obj[isdbk.IS_GROUP_MST_ID_KEY_FEIDB]
        equip_type_mst_ID = facility_equip_input_DB_obj[isdbk.IS_FACILITY_EQUIP_TYPE_MST_ID_KEY_FEIDB]

        # Records are initialized from the equipment database according to module ID and equipment type. All furniture records for the corresponding
        # module are initialized. Medical equipment is initialized for only the default equipment group ID.
        if (((costing_mode == gbc.GB_TB_COSTING_MODE) and (facility_equip_input_DB_obj[isdbk.IS_MOD_ID_KEY_FEIDB] == gbc.GB_TB)) 
            or (((costing_mode != gbc.GB_TB_COSTING_MODE)) and (facility_equip_input_DB_obj[isdbk.IS_MOD_ID_KEY_FEIDB] != gbc.GB_TB))
            and (equip_type_mst_ID == isc.IS_FACIL_FURNITURE_EQUIP_MST_ID or facility_MEG_mst_ID == isc.IS_DEF_MED_EQUIP_GROUP_ID)):
        
            facility_type_mst_ID = facility_equip_input_DB_obj[isdbk.IS_FACILITY_TYPE_MST_ID_KEY_FEIDB]

            if (facility_type_ID == str(gbc.GB_NOT_FOUND)) or (facility_type_ID == facility_type_mst_ID): 

                equip_mst_ID = facility_equip_input_DB_obj[isdbk.IS_EQUIP_MST_ID_KEY_FEIDB]
                equip_name = facility_equip_input_DB_obj[isdbk.IS_EQUIP_NAME_MST_ID_KEY_FEIDB]
                equip_string_constant = facility_equip_input_DB_obj[isdbk.IS_EQUIP_STR_CONST_KEY_FEIDB]
                unit_cost = facility_equip_input_DB_obj[isdbk.IS_UNIT_COST_KEY_FEIDB]
                units_per_facility = facility_equip_input_DB_obj[isdbk.IS_UNITS_PER_FACILITY_TYPE_KEY_FEIDB]

                # Note that the equipment mstID currently isn't used in the record but still passing it in, in case the winds change direction
                facility_equip_record = ismvd.IS_create_facility_equip_record(equip_mst_ID, equip_name, facility_type_mst_ID, equip_type_mst_ID, costing_mode, IHT_plan_type)
                ismvu.set_IS_facility_equip_string_constant(facility_equip_record, equip_string_constant)

                # Only facility medical equipment has a corresponding MEG (medical equipment group).
                if equip_type_mst_ID == isc.IS_FACIL_MEDICAL_EQUIP_MST_ID:
                    ismvu.set_IS_facility_med_equip_MEG_mst_ID(facility_equip_record, facility_MEG_mst_ID)
                
                ismvu.set_IS_facility_equip_unit_cost(facility_equip_record, unit_cost)
                ismvu.set_IS_facility_equip_units_per_facility(facility_equip_record, units_per_facility)

                if equip_type_mst_ID == isc.IS_FACIL_MEDICAL_EQUIP_MST_ID:
                    facility_med_equip_dict_list.append(facility_equip_record)
                else:
                    facility_furniture_dict_list.append(facility_equip_record)

    return facility_med_equip_dict_list, facility_furniture_dict_list

#######################################################################################################
#                                                                                                     #
#                                       Meta data                                                     #
#                                                                                                     #
#######################################################################################################

def IS_init_default_MEG_list_meta(facility_MEG_DB, costing_mode, plan_type):
    default_MEG_list_meta = []

    if (costing_mode == gbc.GB_IHT_COSTING_MODE and plan_type == iscl.IH_SPECIFIC_PLAN):
        modID = iscl.GB_IS
    elif (costing_mode == gbc.GB_TB_COSTING_MODE):
        modID = iscl.GB_TB
    else:
        return []

    for facility_MEG_DB_obj in facility_MEG_DB:
        if (facility_MEG_DB_obj[isdbk.IS_MOD_ID_KEY_FEIDB] == modID):
            default_MEG_list_meta.append({
                ismvf.IS_FACIL_MEG_NAME : facility_MEG_DB_obj[isdbk.IS_GROUP_NAME_ID_KEY_FEIDB],
                ismvf.IS_FACIL_MEG_ID : facility_MEG_DB_obj[isdbk.IS_GROUP_MST_ID_KEY_FEIDB],
                ismvf.IS_FACIL_MEG_FACIL_TYPE_ID : facility_MEG_DB_obj[isdbk.IS_FACILITY_TYPE_MST_ID_KEY_FEIDB],
                ismvf.IS_FACIL_MEG_STR_CONST : facility_MEG_DB_obj[isdbk.IS_GROUP_STR_CONST_KEY_FEIDB]
            })

    return default_MEG_list_meta


# Main transaction record fields

TR_TARGET = "target"  # string; Type of item to perform transaction on.
TR_COMMAND = "command"  # string; Type of transaction to perform.
TR_PARAMS = "params"  # Any; Put all parameters in here

# Params (P) for 'params' field of service call. Declare in TRConstLink and pull from the appropriate
# module if you can so we can stay consistent. If those fields don't exist in modules or we can use a single field for all modules (like in the
# case with 'name' or 'active'), declare them here instead.

TR_P_NAME = "name"  # string
TR_P_ACTIVE = "active"  # boolean
TR_P_INTERV_IDS = "intervIDs"  # list of string
TR_P_PROG_AREA_IDS = "progAreaIDs"  # list of string
TR_P_FACILITY_TYPE_IDS = "facilityTypeIDs"  # list of string
TR_P_GROUP_NAME = "groupName"  # string
TR_P_NEW_GROUP_ID = "newGroupID"  # string
TR_P_TREAT_INPUT_TYPE_MST_ID = "treatInputTypeMstID"  # integer
TR_P_DRUG_SUPPLY_NAME = "drugSupplyName"  # string
TR_P_UNIT_COST = "unitCost"  # float OR list of float
TR_P_UNIT_COST_YEAR = "unitCostYear"  # int
TR_P_FACILITY_EQUIP_TYPE_MST_ID = "facilityEquipTypeMstID"  # int
TR_P_MOD_ID = "modID"  # int
TR_P_VALUE = "value"  # Any
TR_P_FILE_TITLE = "fileTitle"  # string
TR_P_INTERV_DICT = "intervDict"  # dict where keys are intervID's
TR_P_LANGUAGE_ID = "languageID"  # int
TR_P_ADD_IND_TYPE_IDS = "additionalIndTypeIDs"  # list
TR_P_PPLI_PHASES_CHANGED = "PPLIPhasesChanged"  # 2d list of boolean (costing subsection x year)
TR_P_FIRST_YEAR_POP_FIVE_YEAR_AGE_GROUPS = "firstYearPopFiveYearAgeGroups"  # 2d list of int (sex x five year age group)
# TR_P_POLICY_POP_LEVEL_INTERV_ENTRY = 'policyPopLevelIntervEntry' # boolean

# ModVar tag record fields (contained within the TR_MOD_VAR_TAGS field))

TR_MV_MODULE = "module"
TR_MV_TAGS = "modvars"

# Transaction service fields

TR_PROJ_ID = "projID"  # string; returned by CreateProjection
TR_TRANSACTIONS = "transactions"  # list of transaction dicts
TR_CLIENT_MODULE_MOD_VAR_TAGS = "clientModvarList"  # Transaction Server will return all ModVars corresponding to this list of ModVar tags that were changed.

# Targets

TR_PROJECTION = "Projection"
TR_YEARS = "Years"
TR_INTERVENTIONS = "Interventions"
TR_PROG_AREAS = "ProgramAreas"
TR_PROG_AREA_SUBGROUPS = "ProgramAreaSubgroups"
TR_DELIV_CHANNELS = "DeliveryChannels"
TR_DELIV_CHAN_PACKS = "DeliveryChannelPackages"
TR_NAT_PROGRAMS = "NationalPrograms"
TR_DRUGS_SUPPLIES = "DrugsAndSupplies"
TR_FACILITY_TYPES = "FacilityTypes"
TR_FACILITY_EQUIPMENT = "FacilityEquipment"
TR_FACILITY_MED_EQUIP_GROUPS = "FacilityMedicalEquipmentGroups"
TR_VEHICLES_IS = "Vehicles"  # Should be VehicleTypesIS
TR_ICT_EQUIPMENT = "ICT_Equipment"
TR_STAFF_TYPES = "StaffTypes"
TR_ATTRITION_TYPES = "AttritionTypes"
TR_RETENTION_INCENTIVES = "RetentionIncentives"
TR_TREATMENT_INPUTS = "TreatmentInputs"
TR_TREATMENT_INPUT_GROUPS = "TreatmentInputGroups"
TR_TREATMENT_INPUT_CALCS_IC = "TreatmentInputCalcsIC"
TR_POP_AND_COV_CALCS_IC = "PopulationAndCoverageCalcsIC"
TR_COST_PER_CASE_CALC_IC = "CostPerCaseIC"
TR_MIN_PER_AVG_CASE_CALC_IC = "MinutesPerAvgCaseIC"
TR_INPAT_DAYS_OUTPAT_VISITS_PER_CASE_CALC_IC = "InpatDaysOutpatVisitsPerCaseIC"
TR_PIN_CALC_IC = "PopInNeedIC"
TR_TRANSFER_COV_TO_IMPACT_MODS_CALC_IC = "TransferCoveragesToImpactModulesIC"
TR_TRANSFER_COV_TO_IHT_CALC_IC = "TransferCoveragesToIHT_IC"
TR_COST_CATS_PC = "CostCategoriesPC"
TR_COST_INPUTS_PC = "CostInputsPC"
TR_COSTING_SECT_PC = "CostingSectionsPC"
TR_COSTING_SUBSECT_PC = "CostingSubsectionsPC"
TR_ACTIVITIES_PC = "ActivitiesPC"
TR_ACT_LINE_ITEMS_PC = "ActivityLineItemsPC"
TR_TOTAL_EQUIP_COSTS_IS = "TotalEquipmentCostsIS"
TR_BUDGETS = "Budgets"
TR_BUDGET_CATS = "BudgetCategories"
TR_BUDGET_CAT_SHARES = "BudgetCategoryShares"
TR_FUND_FRAMEWORKS = "FundingFrameworks"
TR_FUND_SOURCES = "FundingSources"
TR_FUND_SOURCE_SHARES = "FundingSourceShares"
TR_BUDGET_INDICATORS = "BudgetIndicators"
TR_QUICK_MAPPING = "QuickMapping"
TR_HEALTH_SYS_ADMIN_UNITS = "HealthSystemAdminUnits"
TR_TAXATION_CALC_PC = "TaxationCalcPC"
TR_PHASE_INTENSITY_CALC_PC = "PhaseIntensityCalcPC"
TR_TAX_PHASE_INTENS_CALCS_PC = "TaxationPhaseIntensityCalcsPC"
TR_POP_DP = "PopulationDP"
TR_SUPPLY_CHAIN_LEVELS = "SupplyChainLevels"
TR_WAREHOUSES = "Warehouses"
TR_THIRD_PARTY_LOG_CONTRACTS = "ThirdPartyLogisticsContracts"
TR_VEHICLE_TYPES_LG = "VehicleTypesLG"
TR_WORKER_TYPES_LG = "WorkerTypesLG"
TR_COLD_CHAIN_LG = "ColdChainLG"
TR_CALC_VEHICLE_TARGETS_LG = "CalcVehicleTargetsLG"
TR_CALC_WORKER_TARGETS_LG = "CalcWorkerTargetsLG"
TR_CALC_COLD_CHAIN_TARGETS_LG = "CalcColdChainTargetsLG"
TR_INITIALIZE_VEHICLE_TARGETS_LG = "InitializeVechileTargetsLG"
TR_INITIALIZE_WORKER_TARGETS_LG = "InitializeWorkerTargetsLG"
TR_INITIALIZE_COLD_CHAIN_TARGETS_LG = "InitializeColdChainTargetsLG"
TR_CALC_VEHICLE_TARGETS_ALL_LG = "CalcVehicleTargetsAllLG"
TR_CALC_WORKER_TARGETS_ALL_LG = "CalcWorkerTargetsAllLG"
TR_CALC_COLD_CHAIN_TARGETS_ALL_LG = "CalcColdChainTargetsAllLG"


# Commands, Generic

TR_CREATE = "Create"  # Create/Add new item. (custom only)
TR_CREATE_MULTIPLE = "CreateMultiple"  # Create batches of items in one shot (custom only)
TR_MANAGE_MULTIPLE = "ManageMultiple"  # Creates and/or deletes batches of items in one shot
TR_RENAME = "Rename"  # Change name of item.
TR_DELETE = "Delete"  # Delete item (custom only).
TR_ACTIVATE = "Activate"  # Set one item active.
TR_ACTIVATE_ALL = "ActivateAll"  # Set all items active.
TR_DEACTIVATE = "Deactivate"  # Set one item inactive.
TR_DEACTIVATE_ALL = "DeactivateAll"  # Set all items inactive.
TR_MOVE = "Move"  # Move item(s).
TR_CHANGE = "Change"  # For increasing/decreasing years.
TR_COPY = "Copy"  # Copy items
TR_CALC = "Calculate"  # Calculate something and fill ModVar(s)
TR_SET_TESTING_DEFAULTS = "SetTestingDefaults"  # Set default values for testing a projection.
TR_TEST_TRANSACTIONS = "TestTransactions"  # Run one or more transaction test(s)
TR_GENERATE_FILE = "GenerateFile"  # Generate file.
TR_CHANGE_LANGUAGE = (
    "ChangeLanguage"  # Updates the name of all non-custom server strings depending on the language passed to it.
)
TR_UPDATE_STRINGS = "UpdateStrings"  # Updates all string constants to the latest.
TR_RELOAD_DEFAULTS = "ReloadDefaultValues"  # Reload default values

# Commands, Special/non-generic

TR_SET_STAFF_DIVISION = "SetStaffDivision"  # target: TR_STAFF_TYPES only. Set staff type's staff division
TR_SET_STAFF_DEF_MAP = "SetStaffDefaultMapping"  # target: TR_STAFF_TYPES only. Set staff type's default staff mapping.
TR_SET_FACIL_SUPP_TYPE = "SetFacilitySupportType"  # target: TR_FACILITY_TYPES only. Set facility type's support type.
TR_SET_FACIL_DELIV_CHAN = (
    "SetFacilityDeliveryChannel"  # target: TR_FACILITY_TYPES only. Set facility type's delivery channel.
)
TR_SET_FACIL_DEF_MAP = (
    "SetFacilityDefaultMapping"  # target: TR_FACILITY_TYPES only. Set facility type's default facility mapping.
)
TR_SET_FACIL_EQUIP_TOTAL_COST = (
    "SetFacilityTotalEquipmentCost"  # target: TR_FACILITY_TYPES only. Set total equipment cost.
)
TR_SET_FACIL_MED_EQUIP_LISTS = "SetFacilityMedicalEquipmentLists"  # target: TR_FACILITY_TYPES only. Loads initial facility equipment lists for one or more facilities.
TR_SET_INTERV_PARAMS_MULTIIPLE = "SetInterventionParametersMultiple"  # target: TR_INTERVENTIONS only. Set multiple parameters at once for an intervention.
TR_SET_INTERV_PROG_AREA = (
    "SetInterventionProgramArea"  # target: TR_INTERVENTIONS only. Set intervention's programme area.
)
TR_SET_INTERV_SUBGROUP = "SetInterventionSubgroup"  # target: TR_INTERVENTIONS only. Set intervention's subgroup.
TR_SET_INTERV_ACTIVE_FROM_CS = "SetInterventionsActiveFromCS"  # target: TR_INTERVENTIONS only. Set intervention's active status based on LiST's 'selected' status.
TR_SET_INTERV_ACTIVE_FROM_FP = "SetInterventionsActiveFromFP"  # target: TR_INTERVENTIONS only. Set intervention's active status based on FamPlan's active methods MV.
TR_SET_DRUG_SUPP_PARAMS_MULTIPLE = "SetDrugSupplyParametersMultiple"  # target: TR_DRUGS_SUPPLIES only. Set multiple parameters at once for a drug/supply.
TR_SET_BUDGET_FUNDING_FRAMEWORKS = "SetBudgetFundingFrameworks"  # target: TR_BUDGETS only. Sets the funding framework for all budgets to the one of the budget ID specified.
TR_SET_IND_BUDGET_CAT = "SetIndicatorBudgetCategory"  # target: TR_BUDGET_INDICATORS only.
TR_SET_IND_FUND_SOURCE = "SetIndicatorFundingSource"  # target: TR_BUDGET_INDICATORS only.
TR_SET_BUDGET_CATS_UNALLOCATED = "SetIndicatorBudgetCategoriesUnallocated"  # target: TR_BUDGET_INDICATORS only. Sets all indicator mappings to Unallocated.
TR_SET_FUND_SOURCES_UNALLOCATED = "SetIndicatorFundingSourcesUnallocated"  # target: TR_BUDGET_INDICATORS only. Sets all indicator mappings to Unallocated.
TR_SET_COVERAGES = "SetCoverages"  # target: TR_INTERVENTIONS only. Set one or more intervention coverages.
TR_SET_QUICK_MAPPING = "SetQuickMapping"  # target : TR_QUICK_MAPPING only.  Sets quick mapping on/off
TR_SPLIT_FIRST_YEAR_POP = (
    "SplitFirstYearPop"  # target : TR_POP_DP only. Splits first year pop by five year age groups into single year ages
)
TR_SET_NUM_SUPPLY_CHAIN_LEVELS = "SetNumSupplyChainLevels"  # target:



#######################################################################################################
#                                                                                                     #
#                                       Facility Equipment Inputs DB (FEIDB)                          #
#                                                                                                     #
#######################################################################################################

IS_MOD_ID_KEY_FEIDB                     = '<Module ID>'
IS_MOD_NAME_KEY_FEIDB                   = '<Module Name>'
IS_FACILITY_TYPE_MST_ID_KEY_FEIDB       = '<Facility Type Master ID>'
IS_FACILITY_TYPE_NAME_ID_KEY_FEIDB      = '<Facility Type Name>'
IS_GROUP_MST_ID_KEY_FEIDB               = '<Group Master ID>'
IS_GROUP_NAME_ID_KEY_FEIDB              = '<Group Name>'
IS_GROUP_STR_CONST_KEY_FEIDB            = '<Group String Constant>'
IS_FACILITY_EQUIP_TYPE_MST_ID_KEY_FEIDB = '<Facility Equipment Type Master ID>'
IS_EQUIP_MST_ID_KEY_FEIDB               = '<Equipment Master ID>'
IS_EQUIP_NAME_MST_ID_KEY_FEIDB          = '<Equipment Name>'
IS_EQUIP_STR_CONST_KEY_FEIDB            = '<Equipment String Constant>'
IS_UNIT_COST_KEY_FEIDB                  = '<Unit Cost>'
IS_UNITS_PER_FACILITY_TYPE_KEY_FEIDB    = '<Units per Facility Type>'



FP_ActiveMethodsTag                = '<FP_ActiveMethods_V1>'
# FP_DataChangedTag                  = '<FP_DataChanged_V1>' 
# FP_ProjectionValidTag              = '<FP_ProjectionValid_V1>'
# FP_ProjectionValidDateTag          = '<FP_ProjectionValidDate_V1>'
FP_AgeGroupOptionTag               = '<FP_AgeGroupOption_V1>'
FP_GoalTag                         = '<FP_Goal_V1>'
FP_AbortionOptionTag               = '<FP_AbortionOption_V1>'
FP_NeedOptionTag                   = '<FP_NeedOption_V1>'
FP_SourcesTag                      = '<FP_Sources_V1>'
FP_MethodNameTag                   = '<FP_MethodName_V1>'
FP_NumSourcesTag                   = '<FP_NumSources_V1>'
FP_NumNeedsTag                     = '<FP_NumNeeds_V1>'
FP_NumCustomMethodsTag             = '<FP_NumCustomMethods_V1>'
FP_PregnancyRateTag                = '<FP_PregnancyRate_V1>'
FP_PercentPregWithMiscarriageTag   = '<FP_PercentPregWithMiscarriage_V1>'
FP_PropUnintendedPregEndInAbortTag = '<FP_PropUnintendedPregEndInAbort_V1>'
FP_PropUnsafeAbortTag              = '<FP_PropUnsafeAbort_V1>'
FP_MortalityRateSafeAbortTag       = '<FP_MortalityRateSafeAbort_V1>'
FP_MortalityRateUnsafeAbortTag     = '<FP_MortalityRateUnsafeAbort_V1>'
FP_PropPregWantedLaterTag          = '<FP_PropPregWantedLater_V1>'
FP_PropPregUnwantedTag             = '<FP_PropPregUnwanted_V1>'
FP_MiscarriagePercentTag           = '<FP_MiscarriagePercent_V1>'
FP_CalcMethOptionTag               = '<FP_CalcMethOption_V1>'
FP_MethodMixTag                    = '<FP_MethodMix_V1>'
FP_MethodPermTermTag               = '<FP_MethodPermTerm_V1>'
FP_MethodLongTermTag               = '<FP_MethodLongTerm_V1>'
FP_MethodshortTermTag              = '<FP_MethodshortTerm_V1>'
FP_EffectivenessTag                = '<FP_Effectiveness_V1>'
FP_EffectivenessMetaTag            = '<FP_Effectiveness_Meta_V1>'
FP_UnitsPerCYPTag                  = '<FP_UnitsPerCYP_V1>'
FP_AverageAgeTag                   = '<FP_AverageAge_V1>'
FP_DurationOfUseTag                = '<FP_DurationOfUse_V1>'
FP_SourceMixTag                    = '<FP_SourceMix_V1>'
FP_CostPerUATag                    = '<FP_CostPerUA_V1>'
FP_FeesTag                         = '<FP_Fees_V1>'
FP_GrossCostTag                    = '<FP_GrossCost_V1>'
FP_RevenueTag                      = '<FP_Revenue_V1>'
FP_NetCostTag                      = '<FP_NetCost_V1>'
FP_CommoditiesTag                  = '<FP_Commodities_V1>'
FP_PercentInUnionTag               = '<FP_PercentInUnion_V1>'
FP_PPITag                          = '<FP_PPI_V1>'
FP_TARTag                          = '<FP_TAR_V1>'
FP_AbortionPercentTag              = '<FP_AbortionPercent_V1>'
FP_SterilityTag                    = '<FP_Sterility_V1>'
FP_MarriageIndexTag                = '<FP_MarriageIndex_V1>'
FP_PPIIndexTag                     = '<FP_PPIIndex_V1>'
FP_AbortionIndexTag                = '<FP_AbortionIndex_V1>'
FP_SterilityIndexTag               = '<FP_SterilityIndex_V1>'
FP_PrevalenceIndexTag              = '<FP_PrevalenceIndex_V1>'
FP_TFRTag                          = '<FP_TFR_V1>'
FP_TFRMetaTag                      = '<FP_TFR_Meta_V1>'
FP_WRATag                          = '<FP_WRA_V1>'
FP_MWRATag                         = '<FP_MWRA_V1>'
FP_MortalityRateTag                = '<FP_MortalityRate_V1>'
FP_BirthsTag                       = '<FP_Births_V1>'
FP_BirthsAvertedTag                = '<FP_BirthsAverted_V1>'
FP_AbortionsTag                    = '<FP_Abortions_V1>'
FP_AverageEffectivenessTag         = '<FP_AverageEffectiveness_V1>'
FP_PrevalenceTag                   = '<FP_Prevalence_V1>'
FP_MPrevalenceTag                  = '<FP_MPrevalence_V1>'
FP_AddMdrnUsersPrevTag             = '<FP_AddMdrnUsersPrev_V1>'
FP_UnmetNeedTag                    = '<FP_UnmetNeed_V1>'
FP_UnmetNeedReductionTag           = '<FP_UnmetNeedReduction_V1>'
FP_WantedTFRTag                    = '<FP_WantedTFR_V1>'
FP_TFRdifferenceTag                = '<FP_TFRdifference_V1>'
FP_TestUnmetNeedFromPrevTag        = '<FP_TestUnmetNeedFromPrev_V1>'
FP_CalcCostRegressionTag           = '<FP_CalcCostRegression_V1>'
FP_ASFRInput_TFRGoalTag            = '<FP_ASFRInput_TFRGoal_V1>'
FP_ChildSurvInputsTag              = '<FP_ChildSurvInputs_V1>'
FP_PACInputsTag                    = '<FP_PACInputs_V1>'
FP_LegalAbortNeedTreatTag          = '<FP_LegalAbortNeedTreat_V1>'
FP_PACOutputsTag                   = '<FP_PACOutputs_V1>'
FP_LAMinsusceptibleTag             = '<FP_LAMinsusceptible_V1>'
FP_EPSurveyYearTFRTag              = '<FP_EPSurveyYearTFR_V1>'
FP_TFTag                           = '<FP_TF_V1>'
FP_PRTag                           = '<FP_PR_V1>'
FP_DiscontinuationRateTag          = '<FP_DiscontinuationRate_V1>'
FP_UsersTag                        = '<FP_Users_V1>'
FP_AcceptorsTag                    = '<FP_Acceptors_V1>'
FP_GrowthRateTag                   = '<FP_GrowthRate_V1>'
FP_PregnanciesTag                  = '<FP_Pregnancies_V1>'
FP_ChildSurvOutputsTag             = '<FP_ChildSurvOutputs_V1>'
FP_PregAvertedTag                  = '<FP_PregAverted_V1>'
FP_MaternalDeathsAvertedTag        = '<FP_MaternalDeathsAverted_V1>'
FP_UnsafeAbortionsAvertedTag       = '<FP_UnsafeAbortionsAverted_V1>'
FP_RelativeRisksTag                = '<FP_RelativeRisks_V1>'
FP_RRAgeBirthOrderResultTag        = '<FP_RRAgeBirthOrderResult_V1>'
FP_RRBirthIntervalResultTag        = '<FP_RRBirthIntervalResult_V1>'
# FP_EdFPSourceTag                   = '<FP_EdFPSource_V1>'
# FP_EdGLSourceTag                   = '<FP_EdGLSource_V1>'
FP_SourceTag                       = '<FP_Source_V1>'
FP_DataSourceTag                   = '<FP_DataSource_V1>'
FP_AvgHouseholdTag                 = '<FP_AvgHousehold_V1>'
FP_GoalAnnualChangeTag             = '<FP_GoalAnnualChange_V1>'

##### OLD TAGS GO BELOW #####


# Fields used in IS data structures

'''
    General notes:

    We avoid keeping lists of records inside of records (for example, medical equipment records inside facility records) mainly to keep 
    structures from becoming too large.  This will hopefully prevent slowdowns and memory inefficiency as well as make certain things easier
    to spot manually when looking through records. Use a field (such as facilityTypeID) to connect these records.

    IDs are spelled out like facilityTypeID rather than just ID inside records to make things obvious when manually looking through records.
'''

IS_F_FACILITY_TYPE_ID            = 'facilityTypeID' 
IS_F_FACILITY_MED_EQUIP_GROUP_ID = 'facilityMedEquipGroupID'
IS_F_FACILITY_EQUIP_ID           = 'facilityEquipID' 
IS_F_VEHICLE_TYPE_ID             = 'vehicleID'   
IS_F_ICT_EQUIP_ID                = 'ICT_EquipID' 

# facility records - contains all inputs that are not also lists of records. Those are kept outside mainly for size issues. Use 
# facilityTypeID to connect the facility records to those other lists of records.

IS_FACIL_TYPE_ID                                   = IS_F_FACILITY_TYPE_ID                     # string;  numeric mstID for defaults. GUID for customs.
IS_FACIL_STR_CONSTANT                              = 'stringConstant'                          # string;  Defaults only.
IS_FACIL_NAME                                      = 'name'                                    # string;  Holds custom name or overridden default name.
IS_FACIL_ACTIVE                                    = 'active'                                  # boolean; Is user using this item or not?
IS_FACIL_SUPP_TYPE_MST_ID                          = 'supportTypeMstID'                        # integer;
IS_FACIL_DELIV_CHAN_MST_ID                         = 'delivChanMstID'                          # string;  ID from delivery channel records
IS_FACIL_CUSTOM                                    = 'custom'                                  # boolean; user-entered = true; default = false
IS_FACIL_DEFAULT_MAP_MST_ID                        = 'defFacilityTypeMapMstID'                 # int;  mstID of facility type (from facility type records) user maps custom facility types to
IS_FACIL_EQUIP_PURCHASE_COST                       = 'equipmentPurchaseCost'                   # 1d list of float; by equipment type
IS_FACIL_MED_EQUIP_MAINT_COST                      = 'medEquipmentMaintCost'                   # float; TB app only
IS_FACIL_MED_EQUIP_CALIB_COST                      = 'medEquipmentCalibrationCost'             # float; TB app only
IS_FACIL_NUM_TO_EQUIP                              = 'numToEquip'                              # 1d list of integer; by year
IS_FACIL_TOTAL_NUMBER_BASELINE                     = 'totalNumBaseline'                        # integer
IS_FACIL_AVG_NUM_BEDS                              = 'avgNumBeds'                              # integer
IS_FACIL_AVG_OCCUP_RATE_BEDS                       = 'avgOccupancyRateBeds'                    # integer
IS_FACIL_AVG_NUM_OUTPAT_VISITS_PER_YEAR            = 'avgNumOutpatVisitsPerYear'               # integer
IS_FACIL_YEARS_TO_BUILD                            = 'yearsToBuild'                            # integer
IS_FACIL_TOTAL_CONSTRUCT_COST                      = 'totalConstructCost'                      # integer
IS_FACIL_LAND_COST                                 = 'landCost'                                # float
IS_FACIL_CONSTRUCT_COST_PER_SQ_M                   = 'constructionCostPerSqM'                  # float
IS_FACIL_AVG_SIZE_IN_SQ_M                          = 'avgSizeSqM'                              # float
IS_FACIL_REHAB_COST                                = 'rehabCost'                               # 1d list of float; by rehab type
IS_FACIL_TOTAL_OP_COST                             = 'totalOpCost'                             # float
IS_FACIL_TOTAL_OP_COSTS_PERC_CONSTRUCT_COST        = 'totalOpCostAsPercOfConstructCost'        # float
IS_FACIL_DESIRED_POP_FACIL_RATIO_POP_NORMS         = 'desiredPopPerFacilityRatioPopNorms'      # integer
IS_FACIL_TARG_SCALE_UP_POP_NORMS                   = 'targScaleUpPopNorms'                     # 1d list of integer; by year
IS_FACIL_TARG_SCALE_UP_EXIST_PLANS_EXPECTED_OP     = 'targScaleUpExistingPlansExpectedOp'      # 1d list of integer; by year
IS_FACIL_TARG_SCALE_UP_EXIST_PLANS_BEGIN_CONSTRUCT = 'targScaleUpExistingPlansBeginConstruct'  # 1d list of integer; by year
IS_FACIL_TARG_NUM_TO_REHAB                         = 'targNumToRehab'                          # 1d list of integer; by rehab type
IS_FACIL_TARG_NUM_TO_REHAB_BY_YEAR                 = 'targNumToRehabByYear'                    # 2d list of integer; rehab type x year

# facility medical equipment group records

IS_FACIL_MEG_ID                = IS_F_FACILITY_MED_EQUIP_GROUP_ID # string; numeric mstID for defaults (still cast as string). GUID for custom groups
IS_FACIL_MEG_NAME              = 'name'                           # string
IS_FACIL_MEG_STR_CONST         = 'stringConstant'                 # string;  Defaults only.
IS_FACIL_MEG_FACIL_TYPE_ID     = IS_F_FACILITY_TYPE_ID            # string

# facility equipment records (can be either medical equipment or furniture)

IS_FACIL_EQUIP_ID            = IS_F_FACILITY_EQUIP_ID # string; numermic mstID for defaults (still cast as string). GUID for custom equipment.
IS_FACIL_EQUIP_NAME          = 'name'                 # string
IS_FACIL_EQUIP_STR_CONST     = 'stringConstant'       # string;  Defaults only.
IS_FACIL_EQUIP_CUSTOM        = 'custom'               # boolean
IS_FACIL_EQUIP_FACIL_TYPE_ID = IS_F_FACILITY_TYPE_ID  # string
IS_FACIL_EQUIP_UNIT_COST     = 'unitCost'             # float
IS_FACIL_EQUIP_NUM_UNITS     = 'numUnits'             # float
IS_FACIL_EQUIP_WORKING_LIFE  = 'workingLife'          # float
#IS_FACIL_EQUIP_REQ_PURCHASE = 'reqPurchase'          # float
IS_FACIL_EQUIP_MAINT_COST    = 'maintenanceCost'      # float, medical equipment only, TB app only
IS_FACIL_EQUIP_CALIB_COST    = 'calibrationCost'      # float, medical equipment only, TB app only

# for facility medical equipment records

IS_FACIL_EQUIP_MEG_ID  = IS_F_FACILITY_MED_EQUIP_GROUP_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom groups

# vehicle records

IS_VEHIC_ID                          = IS_F_VEHICLE_TYPE_ID                      # string; numermic mstID for defaults (still cast as string). GUID for custom equipment.
IS_VEHIC_NAME                        = 'name'                               # string
IS_VEHIC_CUSTOM                      = 'custom'                             # bool
IS_VEHIC_CURR_NUMBER                 = 'currNumber'                         # float
IS_VEHIC_UNIT_COST                   = 'unitCost'                           # float
IS_VEHIC_WORKING_LIFE_YEARS          = 'workingLifeInYears'                 # float
IS_VEHIC_DRIVER_SALARY               = 'driverSalary'                       # float
IS_VEHIC_FUEL_CONSUMPTION            = 'fuelConsumption'                    # float
IS_VEHIC_FUEL_COST_DIRECT_ENTRY      = 'fuelCostDirectEntry'                # float
IS_VEHIC_DISTANCE_DRIVEN_PER_YEAR    = 'distanceDrivenPerYear'              # float
IS_VEHIC_REPAIR_MAINT_PERC_CAP_COST  = 'repairAndMaintPercCapCost'          # float
IS_VEHIC_TARG_NUM_PER_FACILITY       = 'targNumPerFacility'                 # 1d list of integer; by facility
IS_VEHIC_TARG_NUM_PER_FACILITY_OTHER = 'targNumPerFacilityOtherNonFacBased' # integer

# ICT equipment records

IS_ICT_EQUIP_ID                      = IS_F_ICT_EQUIP_ID                     # string; numermic mstID for defaults (still cast as string). GUID for custom equipment.
IS_ICT_EQUIP_NAME                    = 'name'                                # string
IS_ICT_EQUIP_CUSTOM                  = 'custom'                              # bool
IS_ICT_EQUIP_CURR_NUMBER             = 'currNumber'                          # float
IS_ICT_EQUIP_UNIT_COST               = 'unitCost'                            # float
IS_ICT_EQUIP_WORKING_LIFE_YEARS      = 'workingLifeInYears'                  # float
IS_ICT_EQUIP_OP_COST_PERC_CAP_COST   = 'opCostPercCapCost'                   # float
IS_ICT_REPAIR_MAINT_PERC_CAP_COST    = 'repairAndMaintPercCapCost'           # float
IS_ICT_TARG_NUM_PER_FACILITY         = 'targNumPerFacility'                  # 1d list of integer; by facility
IS_ICT_TARG_NUM_PER_FACILITY_OTHER   = 'targNumPerFacilityOtherNonFacBased'  # integer


from os import environ
from .CSModvarDef import *
from SpectrumCommon.Const.CS import *
from SpectrumCommon.Const.GB.GBConst import GB_IHT_Client_lc
from SpectrumCommon.Const.IV.IVConst import IV_INTERVENTION_DB_NAME, IV_CS_INTERVENTION_DB_CURR_VERSION, IV_GROUP_DB_NAME, IV_CS_GROUP_DB_CURR_VERSION
from SpectrumCommon.Util.CS.CSDataUtil import *
from AvenirCommon.Database import GB_get_db_json, GB_file_exists

def updateFname(DBName, countryID, Region):
    FName = DBName + '_' + latest_DB_Version[DBName]
    
    if DBName == RegionData: 
        FName += '/' + Region
    
    elif DBName in DBC_DB_Names: 
        FName += '/' + countryID
    
    return FName    

def downloadDatabase(connection, countryID, DBName, Region = ''):    
    FName = updateFname(DBName, countryID, Region) + '.JSON'
    
    if GB_file_exists(connection, 'list', FName):
        return GB_get_db_json(connection, 'list', FName)

def init_modvars(countryID, first_year, final_year, extra):     
    connection = environ[GB_SPECT_MOD_DATA_CONN_ENV]

    CSModvars = {}

    GSKMode = extra[GB_GSKMode]
    exploreLiST = extra['exploreLiST']
    level = extra['level']
    source = extra['source']
    appID = extra['appID']

    DBCName = DataByCountry
    if exploreLiST:
        DBCName = DBC_DataPoints
    
    latest_DB_Version[IV_INTERVENTION_DB_NAME] = IV_CS_INTERVENTION_DB_CURR_VERSION
    latest_DB_Version[IV_GROUP_DB_NAME] = IV_CS_GROUP_DB_CURR_VERSION

    Intervention_DB   = downloadDatabase(connection, countryID, IV_INTERVENTION_DB_NAME)
    DefaultData_DB    = downloadDatabase(connection, countryID, DefaultData)
    IVDefaultData_DB  = downloadDatabase(connection, countryID, IVDefaultData)
    IVGroup_DB        = downloadDatabase(connection, countryID, IV_GROUP_DB_NAME)
    GNI_Per_Cap_DB    = downloadDatabase(connection, countryID, GNI_Per_Cap)
    DataByCountry_DB  = downloadDatabase(connection, countryID, DBCName)
    Readiness_DB      = downloadDatabase(connection, countryID, Readiness)
    CSection_Cov_DB   = downloadDatabase(connection, countryID, CSection_Cov)
    DiarrShigella_DB  = downloadDatabase(connection, countryID, DiarrShigella)
    
    efficacy_choice = extra.get('EfficacyDatabase', CS_DefaultJHUFlag)

    # if IHT client and no efficacy database specified, default to WHO
    if (appID == GB_IHT_Client_lc) and ('EfficacyDatabase' not in extra):
        efficacy_choice = CS_DefaultWHOFlag

    # Download data if WHO efficacy database is selected
    if efficacy_choice == CS_DefaultWHOFlag:
        DefaultData_DB   = downloadDatabase(connection, countryID, DefaultWHOData)
        IVDefaultData_DB = downloadDatabase(connection, countryID, IVDefaultWHOData)

    
    RegionData_DB = downloadDatabase(connection, countryID, RegionData, DataByCountry_DB['Region']) 
    easyFPData = GB_get_db_json(connection, 'famplan', 'countries/' + countryID + '_V1.JSON')

    final_year_idx = final_year - first_year

    DataByCountry_DB['exploreLiST'] = exploreLiST
    DataByCountry_DB['ProjFirstYear'] = first_year
    DataByCountry_DB['ProjFinalYearIdx'] = final_year_idx

    DevMode = False
    
    IVInfo, numIntervs = IVInfo_Init(Intervention_DB, DevMode, GSKMode)
    
    isSubnatWizProj = not level == ""
    
    dataSource = source if (isSubnatWizProj) else DataByCountry_DB[TG_DataSource]
        
    CSModvars = {
        CS_DebugTag                              : [0.0],
        # Subnational Wizard
        CS_SubnatWizProjTag                      : Simple_Value_Init(isSubnatWizProj),
        CS_SubnatRegionTag                       : Simple_Value_Init(level),
        CS_FirstTimeEnteringCoverageTag          : Simple_Value_Init(True),
        CS_SubnatDefDataMstIDsetTag              : Simple_Value_Init([]),
        CS_SubnatDataSourceArrayTag              : Simple_Value_Init({}),

        CS_DataSourceTag	                     : Simple_Value_Init(dataSource),
        
        # Meta data
        CS_IDMapTag                              : IDMap_init(),
        CS_FinalIndicesTag                       : FinalIndices_init(),
        
        # Intervention info
        CS_NumIntervsTag                         : Simple_Value_Init(numIntervs),
        CS_IVInfoTag                             : IVInfo,
        CS_IVGroupInfoTag                        : IVGroupInfo_Init(IVGroup_DB),  
        CS_IVSubgroupInfoTag                     : IVSubgroupInfo_Init(IVGroup_DB),            
        
        # General
        CS_DevModeTag                            : Simple_Value_Init(DevMode),
        CS_FirstYearTag                          : Simple_Value_Init(first_year),
        CS_FinalYearTag                          : Simple_Value_Init(final_year),
        CS_ProjValidTag	                         : Simple_Value_Init(False),
        CS_ProjValidDateTag	                     : Simple_Value_Init('Unavailable'),
        CS_DefaultDBChoiceTag                    : Simple_Value_Init(efficacy_choice),

        # Controls
        CS_CovBYTag                              : Simple_Value_Init(0),
        CS_MenARecommendedTag                    : MenARecommended_Init(DataByCountry_DB),                         
        CS_IPTpCBCheckedTag                      : IPTpCBChecked_Init(DataByCountry_DB),                          
        CS_HaveCalculatedChDeathsByCauseTag      : Simple_Value_Init(False),                          
        CS_FirstYearAtWorkTag                    : Simple_Value_Init(CS_DefMinFirstYrAtWork),
        CS_LastYearAtWorkTag                     : Simple_Value_Init(CS_DefMaxFinalYrAtWork),
        CS_AbortCalcFamPlanTag                   : Simple_Value_Init(True),
        CS_CovEntryModeTag                       : Simple_Value_Init(CS_DirectEntryMode if (appID == GB_IHT_Client_lc) else CS_QualityMode),      
        CS_CalcCovFromInputsTag                  : Simple_Value_Init([CS_ThermalCare_CCFI, CS_InjectAntib_CCFI, CS_VitAMeasTreat_CCFI]),
        CS_EnterBFImproveDataTag                 : Simple_Value_Init(CS_MstBFPromotionMode), # 2025-09-02 All apps now start in promotion "mode". # CS_MstBFPrevMode if not extra[GB_IHT_ON] else CS_MstBFPromotionMode),
        CS_BFPromoCBsTag                         : Simple_Value_Init([False, True, False]),
        CS_StuntMatrixOnTag                      : Simple_Value_Init(False),
        CS_WasteMatrixOnTag                      : Simple_Value_Init(False),
        CS_UseDetailedStuntMatrixTag             : Simple_Value_Init(True),
        CS_UseDetailedWasteMatrixTag             : Simple_Value_Init(True),
        CS_FertilityRisksMatrixOnTag             : Simple_Value_Init(False),
        CS_DirectEntryFITag                      : Simple_Value_Init(False),
        CS_SelectedIVSetTag                      : Simple_Value_Init([]),

        # Health status
        CS_NutrDeficTag                          : NutrDefic_Init(DataByCountry_DB),
        CS_VitADefInPWTag                        : VitADefInPW_Init(DataByCountry_DB),
        CS_StatusAtBirthTag                      : StatusAtBirth_Init(DataByCountry_DB),
        CS_DiarrIncTag                           : Incidence_Init(DataByCountry_DB, TG_IncDiarrhea),
        CS_SevereDiarrIncTag                     : Incidence_Init(DataByCountry_DB, TG_IncSevereDiarrhea),
        CS_PneumIncTag                           : Incidence_Init(DataByCountry_DB, TG_IncPneum),
        CS_MeninIncTag                           : Incidence_Init(DataByCountry_DB, TG_IncMening),
        CS_MaternalAnemiaTag                     : MaternalAnemia_Init(DataByCountry_DB),
        CS_PercWom15t49LowBMITag                 : PercWom15t49LowBMI_Init(DataByCountry_DB),
        CS_SyphilisAmongWRATag                   : SyphilisAmongWRA_Init(DataByCountry_DB),
        CS_PercExposedFalciparumTag              : PercExposedFalciparum_Init(DataByCountry_DB),
        CS_PercWomenWithPrevPreEclampTag         : PercWomenWithPrevPreEclamp_Init(DataByCountry_DB),
        CS_PercWomenWithChronicHypertensionTag   : PercWomenWithChronicHypertension_Init(DataByCountry_DB),
        CS_FolicAcidDeficPrevTag                 : FolicAcidDeficPrev_Init(DataByCountry_DB),
        CS_CongenitalDueToNTDsAFTag              : CongenitalDueToNTDsAF_Init(DataByCountry_DB),
        CS_StuntDistrTag                         : StuntWasteDistr_Init(DataByCountry_DB, TG_StuntingDist),
        CS_WasteDistrTag                         : StuntWasteDistr_Init(DataByCountry_DB, TG_WastingDist),
        CS_SingleStuntDistrTag                   : SingleStuntWasteDistr_Init(final_year_idx),
        CS_SingleWasteDistrTag                   : SingleStuntWasteDistr_Init(final_year_idx),
        CS_DiarrPathTag	                         : DiarrPath_Init(DataByCountry_DB),
        CS_PneumPathTag                          : PneumPath_Init(DataByCountry_DB),
        CS_MeninPathTag	                         : MeninPath_Init(DataByCountry_DB),
        CS_BaselineMortTag                       : BaselineMort_Init(DataByCountry_DB, exploreLiST, final_year_idx),
        CS_PercDthByCauseTag	                 : PercDthByCause_Init(DataByCountry_DB, exploreLiST, final_year_idx),
        CS_AdolMortRatioTag                      : AdolMortRatio_Init(DataByCountry_DB, exploreLiST, final_year_idx),
        CS_PercAdolDeathsByCauseTag	             : PercAdolDeathsByCause_Init(DataByCountry_DB, exploreLiST, final_year_idx),
        CS_MatMortRatioTag                       : MatMortRatio_Init(DataByCountry_DB, exploreLiST, final_year_idx),
        CS_PercMatDthByCauseTag                  : PercMatDthByCause_Init(DataByCountry_DB, exploreLiST, final_year_idx),
        CS_PercPregEndAbortTag                   : PercPregEndAbort_Init(DataByCountry_DB),
        CS_AbortIncRatioTag                      : AbortIncRatio_Init(DataByCountry_DB),
        CS_SBRateTag                             : SBRate_Init(DataByCountry_DB, exploreLiST, final_year_idx),
        CS_SBCausesTag                           : SBCauses_Init(DataByCountry_DB, exploreLiST, final_year_idx),
        CS_PerCapitaIncTag                       : PerCapitaInc_Init(DataByCountry_DB),
        CS_FoodInsecureTag                       : FoodInsecure_Init(DataByCountry_DB),
        CS_AvgHouseholdSizeTag	                 : AvgHouseholdSize_Init(DataByCountry_DB),
        CS_IncludeCostForBCRTag                  : IncludeCostForBCR_Init(),
        CS_IncludeBenefitForBCRTag               : IncludeBenefitForBCR_Init(),
        CS_BaselineGDPTag                        : BaselineGDP_Init(DataByCountry_DB),
        CS_AnnualPercGrowthRateTag	             : AnnualPercGrowthRate_Init(DataByCountry_DB, GNI_Per_Cap_DB, final_year_idx),
        CS_LaborForceParRateTag	                 : LaborForceParRate_Init(DataByCountry_DB, final_year_idx),
        CS_FemaleLaborForceParRateTag	         : FemaleLaborForceParRate_Init(DataByCountry_DB, final_year_idx),
        CS_PercGDPAllocWageTag	                 : PercGDPAllocWage_Init(DataByCountry_DB, final_year_idx),
        CS_DiscountRatesTag                      : DiscountRates_Init(),
        CS_FactorsTag                            : Factors_Init(),

        # Coverage
        CS_CoverageTag                           : Coverage_Init(IVInfo, numIntervs, DataByCountry_DB, Readiness_DB, CSection_Cov_DB),
        CS_BFPrevTag                             : BFPrev_Init(DataByCountry_DB),
        CS_EarlyInitBFPrevTag                    : EarlyInitBFPrev_Init(DataByCountry_DB),
        CS_RoutDetVaccCovTag                     : RoutDetVaccCov_Init(IVInfo, DataByCountry_DB),
        CS_PentaDetVaccCovTag                    : PentaDetVaccCov_Init(final_year_idx),
        CS_VaccineSuppPercTag                    : VaccineSuppPerc_Init(final_year_idx),
        CS_AgeAndBirthOrderMatrixTag	         : AgeAndBirthOrderMatrix_Init(final_year_idx, easyFPData),
        CS_BirthIntervalsMatrixTag               : BirthIntervalsMatrix_Init(final_year_idx, easyFPData),

        # Maternal, Stillbirth, and Child Efficacy
        CS_MatEffTag                             : MatEff_Init(IVInfo, numIntervs, IVDefaultData_DB, DataByCountry_DB),
        CS_SBEffTag                              : SBEff_Init(IVInfo, numIntervs, IVDefaultData_DB, DataByCountry_DB),
        CS_ChildEffTag                           : ChildEff_Init(IVInfo, numIntervs, DefaultData_DB, DataByCountry_DB),
        CS_DetVaccEffTag                         : DetVaccEff_Init(IVInfo, DefaultData_DB, DataByCountry_DB),

        # Effectiveness of interventions on incidence
        CS_DiarrIncReductionTag                  : DiarrIncReduction_Init(DefaultData_DB, DataByCountry_DB),
        CS_SevDiarrIncReductionTag               : SevDiarrIncReduction_Init(DataByCountry_DB),
        CS_PneumIncReductionTag                  : PneumIncReduction_Init(DefaultData_DB, DataByCountry_DB),
        CS_MeninIncReductionTag                  : MeninIncReduction_Init(DefaultData_DB, DataByCountry_DB),
        CS_RelRisksDiarrTag                      : RelRisks_Init(DefaultData_DB, TG_RelRiskDiarrBF),
        CS_RelRisksPneumTag          	         : RelRisks_Init(DefaultData_DB, TG_RelRiskPneumBF),
        CS_RelRisksMeninTag	                     : RelRisks_Init(DefaultData_DB, TG_RelRiskMeninBF),

        # Herd effectiveness of vaccines
        CS_HerdVaccEffTag                        : HerdVaccEff_Init(DataByCountry_DB),

        # Impact on food security
        CS_ImpOnFoodSecurityTag                  : ImpOnFoodSecurity_Init(),

        # Effectiveness of nutrition interventions
        CS_AgeAndBirthOrderTag                   : AgeAndBirthOrder_Init(DefaultData_DB),
        CS_BirthIntervalsTag	                 : BirthIntervals_Init(DefaultData_DB),
        CS_IntervForBOTag                        : IntervForBO_Init(DefaultData_DB, DataByCountry_DB),
        CS_ImpactOnStuntTag                      : ImpactOnStunt_Init(DefaultData_DB),
        CS_ImpactOnWasteTag                      : ImpactOnWaste_Init(DefaultData_DB),
        CS_ImpactOnMatAnemiaTag  	             : ImpactOnMatAnemia_Init(DefaultData_DB),
        CS_ImpactBFPromoTag                      : ImpactBFPromo_Init(DefaultData_DB),
        CS_KMConBFTag                            : KMConBF_Init(DefaultData_DB),
        CS_LBWFTag                               : LBWF_Init(DataByCountry_DB),
        CS_ReducStuntTag                         : ReducStunt_Init(DefaultData_DB),
        CS_EducAttainTag                         : EducAttain_Init(DefaultData_DB, DataByCountry_DB),

        # Impact of under-nutrition on mortality
        CS_RelRisksStuntTag                      : RelRisksStuntWast_Init(DefaultData_DB, DataByCountry_DB, TG_RelRiskStunting),
        CS_RelRisksWasteTag                      : RelRisksStuntWast_Init(DefaultData_DB, DataByCountry_DB, TG_RelRiskWasting),
        CS_RelRisksIUGR_LBWTag	                 : RelRisksIUGR_LBW_Init(DataByCountry_DB),
        CS_ImpactBFOnMortNeoNatTag               : ImpactBFOnMortNeoNat_Init(DefaultData_DB),
        CS_ImpactBFOnMortPostNatTag              : ImpactBFOnMortPostNat_Init(DefaultData_DB),
        CS_RelRisksAnemiaTag                     : RelRisksAnemia_Init(DefaultData_DB, DataByCountry_DB),

        # ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        # /////                                                                                                          /////
        # /////                                         Result Modvars                                                   /////
        # /////                                                                                                          /////
        # ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        CS_PercDeathsAvtdSeqPTag                 : CauseByAgeByYear_Init(final_year_idx),
        CS_MortalityRatesTag                     : MortalityRates_Init(final_year_idx),
        CS_DistPathDiarIncResultTag              : DiarrPathsByAgeByYear_Init(final_year_idx),
        
        # Neonatal, child sectors
        CS_ChDeathsTag                           : AgeByYear_Init(final_year_idx),
        CS_ChDeathsByYearTag                     : ChDeathsByYear_Init(final_year_idx),
        CS_ChDeathsPrevdTag                      : AgeByYear_Init(final_year_idx),
        CS_ChDeathsByCauseTag                    : CauseByAgeByYear_Init(final_year_idx),
        CS_DeathsByCauseByBirthCohortTag         : DeathsByCauseByBirthCohort_Init(final_year_idx),
        CS_ChDeathsPrevdByCauseTag               : CauseByAgeByYear_Init(final_year_idx),
        CS_ChDeathsPrevdByIntTag                 : IntervsByAgeByYear_Init(numIntervs, final_year_idx),
        CS_ChDeathsPrevByInterByCauseTag         : ChDeathsPrevByInterByCause_Init(numIntervs, final_year_idx),
        CS_DetVaccCovResultsTag                  : DetVaccCovResults_Init(final_year_idx),
        CS_DAvtdByVaccByCohortTag                : DAvtdByVaccByCohort_Init(final_year_idx),
        CS_DAvtdByVaccByCohortByCauseTag	     : DAvtdByVaccByCohortByCause_Init(final_year_idx),

        # Maternal sector
        CS_MatDeathsTag                          : Year_Init(final_year_idx),
        CS_MatDeathsPrevTag                      : Year_Init(final_year_idx),
        CS_MatDeathsByCauseTag                   : MatDeathCausesByYear_Init(final_year_idx),
        CS_MatDeathsPrevByCauseTag               : MatDeathCausesByYear_Init(final_year_idx),
        CS_MatDeathsPrevByInterTag               : IntervsByYear_Init(numIntervs, final_year_idx),
        CS_MatDeathsPrevByInterByCauseTag	     : MatDeathsPrevByInterByCause_Init(numIntervs, final_year_idx),
        CS_NumAbortionsTag	                     : NumAbortions_Init(final_year_idx),

        # Stillbirth sector
        CS_SBResultsTag                          : Year_Init(final_year_idx),
        CS_SBPrevResultsTag                      : Year_Init(final_year_idx),
        CS_SBByCauseResultsTag                   : SBCausesByYear_Init(final_year_idx),
        CS_SBPrevByCauseResultsTag               : SBCausesByYear_Init(final_year_idx),
        CS_SBPrevByInterResultsTag               : IntervsByYear_Init(numIntervs, final_year_idx),
        CS_SBPrevByInterByCauseTag               : SBPrevByInterByCause_Init(numIntervs, final_year_idx),

        # Mortality rates sector
        CS_SBRateResultTag                       : Year_Init(final_year_idx),
        CS_MatMortRatioResultTag                 : Year_Init(final_year_idx),
        CS_MatMortRateResultTag                  : Year_Init(final_year_idx),
        CS_MortalityRateResultTag                : MortalityRateResult_Init(final_year_idx),
        CS_RedMortIntervResultTag                : RedMortIntervResult_Init(numIntervs),
        CS_RedMortIntervSortedIDsTag             : RedMortIntervSortedIDs_Init(numIntervs),

        # Nutrition sector
        CS_ShutDownBOAttriByIVTag                : Simple_Value_Init(False),
        CS_SubOpBOAvertByInterTag	             : SubOpBOAvertByInter_Init(numIntervs, final_year_idx),
        CS_MAM_PINTag                            : MAM_PIN_Init(),
        CS_SAM_PINTag                            : AgeByYear_Init(final_year_idx),
        CS_PInVarHtForAgeStuntTag	             : StdDevByAgeByYear_Init(final_year_idx),
        CS_PInVarWtForHtWasteTag                 : StdDevByAgeByYear_Init(final_year_idx),
        CS_PInVarWtForHtWasteCFTag               : StdDevByAgeByYear_Init(final_year_idx),
        CS_NumStuntedTag                         : AgeByYear_Init(final_year_idx),
        CS_NumStuntedAvertedTag	                 : AgeByYear_Init(final_year_idx),
        CS_NumStuntedAvertedByInterTag	         : IntervsByAgeByYear_Init(numIntervs, final_year_idx),
        CS_PStuntRedAccountByIntBaseYrTag        : IntervsByAgeByYear_Init(numIntervs, final_year_idx),
        CS_AdditionalYrsSchoolingGainedTag	     : Year_Init(final_year_idx),
        CS_PercIncEarnStuntDeclineTag	         : Year_Init(final_year_idx),
        CS_ImpLifeEarnDueToStuntReducTag         : DiscountRate_Init(final_year_idx),
        CS_LifetimeEarningsObtainedTag	         : DiscountRate_Init(final_year_idx),
        CS_BFPrevResultTag                       : BFPrevResult_Init(final_year_idx),
        CS_BirthTermResultTag	                 : BirthTermResult_Init(final_year_idx),
        CS_PercAnemicWomenTag                    : AnemiaByYear_Init(final_year_idx),
        CS_NumAnemicWomenTag                     : AnemiaByYear_Init(final_year_idx),
        CS_NumAnemiaCasesPrevTag                 : AnemiaByYear_Init(final_year_idx),
        CS_PrevFoodInsecurityTag                 : Year_Init(final_year_idx),

        # Incidence and etiology sector
        CS_DiarrIncResultTag                     : AgeByYear_Init(final_year_idx),
        CS_SevereDiarrIncResultTag               : AgeByYear_Init(final_year_idx),
        CS_PneumIncResultTag                     : AgeByYear_Init(final_year_idx),
        CS_MeninIncResultTag	                 : AgeByYear_Init(final_year_idx),
        CS_NumDiarrCasesTag                      : AgeByYear_Init(final_year_idx),
        CS_NumSevereDiarrCasesTag                : AgeByYear_Init(final_year_idx),
        CS_NumPneumCasesTag                      : AgeByYear_Init(final_year_idx),
        CS_NumMeninCasesTag                      : AgeByYear_Init(final_year_idx),
        CS_DiarrCasesAvertByInterTag             : DiarrCasesAvertByInter_Init(final_year_idx),
        CS_SevereDiarrCasesAvertByInterTag       : DiarrIntervsByAgeByYear_Init(final_year_idx),
        CS_PneumCasesAvertByInterTag             : PneumCasesAvertByInter_Init(final_year_idx),
        CS_MeninCasesAvertByInterTag             : MeninCasesAvertByInter_Init(final_year_idx),
        CS_PercDiarrDeathsByPathTag	             : DiarrPathsByAgeByYear_Init(final_year_idx),
        CS_PercPneumDeathsByPathTag              : PneumPathsByAgeByYear_Init(final_year_idx),
        CS_PercMeninDeathsByPathTag              : MeninPathsByAgeByYear_Init(final_year_idx),
        CS_DiarrDeathsByPathTag                  : DiarrPathsByAgeByYear_Init(final_year_idx),
        CS_PneumDeathsByPathTag                  : PneumPathsByAgeByYear_Init(final_year_idx),
        CS_MeninDeathsByPathTag                  : MeninPathsByAgeByYear_Init(final_year_idx),
        CS_DiarrDeathsByPathBy1yrAgeGrpsTag      : DiarrPathsBy1YrAgeGrpsByYear_Init(final_year_idx),
        CS_PneumDeathsByPathBy1yrAgeGrpsTag      : PneumPathsBy1YrAgeGrpsByYear_Init(final_year_idx),
        CS_MeninDeathsByPathBy1yrAgeGrpsTag      : MeninPathsBy1YrAgeGrpsByYear_Init(final_year_idx),
        CS_PercDiarrCasesByPathTag	             : DiarrPathsByAgeByYear_Init(final_year_idx),
        CS_PercSevDiarrCasesByPathTag	         : DiarrPathsByAgeByYear_Init(final_year_idx),
        CS_PercPneumCasesByPathTag               : PneumPathsByAgeByYear_Init(final_year_idx),
        CS_PercMeninCasesByPathTag               : MeninPathsByAgeByYear_Init(final_year_idx),
        CS_DiarrCasesByPathTag                   : DiarrPathsByAgeByYear_Init(final_year_idx),
        CS_SevereDiarrCasesByPathTag             : DiarrPathsByAgeByYear_Init(final_year_idx),
        CS_PneumCasesByPathTag                   : PneumPathsByAgeByYear_Init(final_year_idx),
        CS_MeninCasesByPathTag                   : MeninPathsByAgeByYear_Init(final_year_idx),
        CS_DiarrCasesByPathBy1YrAgeGrpsTag       : DiarrPathsBy1YrAgeGrpsByYear_Init(final_year_idx),
        CS_SevereDiarrCasesByPathBy1YrAgeGrpsTag : DiarrPathsBy1YrAgeGrpsByYear_Init(final_year_idx),
        CS_PneumCasesByPathBy1YrAgeGrpsTag       : PneumPathsBy1YrAgeGrpsByYear_Init(final_year_idx),
        CS_MeninCasesByPathBy1YrAgeGrpsTag       : MeninPathsBy1YrAgeGrpsByYear_Init(final_year_idx),
        CS_DiarrCasesByBirthCohTag               : Year_Init(final_year_idx),
        CS_PneumCasesByBirthCohTag               : Year_Init(final_year_idx),
        CS_MeninCasesByBirthCohTag               : Year_Init(final_year_idx),
        CS_SevDiarrCasesByBirthCohTag            : Year_Init(final_year_idx),
        CS_DiarCasesAvtdByYr1stDoseVaccTag       : DetVaccByAgeByYear_Init(final_year_idx),
        CS_PneumCasesAvtdByYr1stDoseVaccTag      : DetVaccByAgeByYear_Init(final_year_idx),
        CS_MeninCasesAvtdByYr1stDoseVaccTag      : DetVaccByAgeByYear_Init(final_year_idx),
        CS_SevDiarrCasesAvtdByYr1stDoseVaccTag   : DetVaccByAgeByYear_Init(final_year_idx),
        CS_DiarrDeathsByPathBirthCohTag          : DiarrPathsByYear_Init(final_year_idx),
        CS_PneumDeathsByPathBirthCohTag          : PneumPathsByYear_Init(final_year_idx),
        CS_MeninDeathsByPathBirthCohTag          : MeninPathsByYear_Init(final_year_idx),
        CS_DiarrCasesByPathBirthCohTag           : DiarrPathsByYear_Init(final_year_idx),
        CS_PneumCasesByPathBirthCohTag           : PneumPathsByYear_Init(final_year_idx),
        CS_MeninCasesByPathBirthCohTag           : MeninPathsByYear_Init(final_year_idx),
        CS_SevDiarrCasesByPathBirthCohTag        : DiarrPathsByYear_Init(final_year_idx),
        CS_DiarrDeathsByPathBirthCohVCTag        : DiarrPathsByYear_Init(final_year_idx),
        CS_PneumDeathsByPathBirthCohVCTag        : PneumPathsByYear_Init(final_year_idx),
        CS_MeninDeathsByPathBirthCohVCTag        : MeninPathsByYear_Init(final_year_idx),
        CS_DiarrCasesByPathBirthCohVCTag         : DiarrPathsByYear_Init(final_year_idx),
        CS_PneumCasesByPathBirthCohVCTag         : PneumPathsByYear_Init(final_year_idx),
        CS_MeninCasesByPathBirthCohVCTag         : MeninPathsByYear_Init(final_year_idx),
        CS_PersonsVaccByIntervTag                : PersonsVaccByInterv_Init(final_year_idx),
        CS_PersonsVaccByIntervBy1YrAgeGrpsTag    : DetVaccByDoseBy1YrAgeGrpsByYear_Init(final_year_idx),

        CS_DiscountedCostsTag                    : Year_Init(final_year_idx),
        CS_DiscountedSocialBenefitsTag           : Year_Init(final_year_idx),
        CS_DiscountedEconBenefitsTag             : Year_Init(final_year_idx),
        CS_IncLifetimeEarningsDueToStuntTag      : Year_Init(final_year_idx),
        
        CS_TotalDiscountedCostsTag               : Simple_Value_Init(0),
        CS_TotalDiscountedSocialBenefitsTag      : Simple_Value_Init(0),
        CS_TotalDiscountedEconBenefitsTag        : Simple_Value_Init(0),
        CS_LifetimeEarningsIncTag                : Simple_Value_Init(0),
        CS_BCRTag                                : Simple_Value_Init(0),
    }

    if GSKMode:
        SetDiarrShigellaDistr(CSModvars, DiarrShigella_DB)
    
    if not exploreLiST:
        SetRegionalValues(CSModvars, RegionData_DB, final_year_idx)
    
    if CSModvars[CS_SubnatWizProjTag]["value"]:
        dataSource = dataSource.split(" ")
        source = dataSource[0]
        surveyYear = dataSource[1]
        SubnatData_DB = downloadDatabase(connection, countryID, SubnatData)[level][surveyYear][source]
        SubnatCov_Init(CSModvars, SubnatData_DB, final_year_idx)

    ModifyCoverage(CSModvars, final_year_idx)    
    ModifyEffectivenessFromHealthStatus(CSModvars)

    Meta_MV_Init(CSModvars)

    return CSModvars


import numpy as np

import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.LG.LGConst as lgc
import SpectrumCommon.Const.LG.LGModVarFields as lgmvf
import SpectrumCommon.Modvars.LG.LGModVarUtil as lgmvu
import SpectrumCommon.Const.LG.LGDatabaseKeys as lgdbk
import SpectrumCommon.Const.LG.LGTags as lgmvt
import SpectrumCommon.Const.PJ.PJNTags as pjmvt


#######################################################################################################
#                                                                                                     #
#                                   Supply chain level records                                        #
#                                                                                                     #
#######################################################################################################


def LG_create_supply_chain_level_record(ID: int, name: str, string_const: str, num_years: int):
    return {
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_ID: ID,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NAME: name,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_STR_CONST: string_const,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_CUSTOM: False,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_AMOUNT: 0,
        # Baseline
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_TOTAL_OPERATING_COSTS: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_ANNUAL_WATER_COST: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_ANNUAL_ELECTRICAL_COST: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_OTHER_OPERATING_COSTS: 0,
        # Target setting
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_TOTAL_CONSTRUCT_COSTS: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_LAND_COST: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_COST_CONSTRUCT_SQ_M: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_SIZE_SQ_M: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NUM_BUILT_BY_YEAR: [0] * num_years,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NUM_REHAB_BY_YEAR: [0] * num_years,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_REHAB_COST: 0,
        # Volume analytics
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NUM_RESUPPLY_VISITS_BY_YEAR: [0] * num_years,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_CAPACITY: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_NUM_MONTHS_SAFETY_STOCK: 0,
        lgmvf.LG_SUPPLY_CHAIN_LEVEL_DISTANCE_TO_SUPPLYING_WAREHOUSE: 0,
    }


def LG_init_supply_chain_level_records(num_years: int):
    supply_chain_level_records = []

    for default_supply_chain_level in lgc.LG_DEFAULT_SUPPLY_CHAIN_LEVELS:
        default_level_rec = LG_create_supply_chain_level_record(
            default_supply_chain_level[lgmvf.LG_SUPPLY_CHAIN_LEVEL_ID],
            default_supply_chain_level[lgmvf.LG_SUPPLY_CHAIN_LEVEL_NAME],
            default_supply_chain_level[lgmvf.LG_SUPPLY_CHAIN_LEVEL_STR_CONST],
            num_years,
        )
        supply_chain_level_records.append(default_level_rec)

    return supply_chain_level_records


#######################################################################################################
#                                                                                                     #
#                                   Warehouse records                                                 #
#                                                                                                     #
#######################################################################################################


def LG_create_warehouse_record(
    ID: str,
    name: str,
    parent_ID: str,
    supply_chain_level_ID: int,
    num_supply_chain_levels: int,
    warehouse_type: int,
    num_years: int,
    custom: bool = True,
):
    return {
        lgmvf.LG_WAREHOUSE_ID: ID,
        lgmvf.LG_WAREHOUSE_NAME: name,
        lgmvf.LG_WAREHOUSE_PARENT_ID: parent_ID,
        lgmvf.LG_WAREHOUSE_SUPPLY_CHAIN_LEVEL_ID: supply_chain_level_ID,
        lgmvf.LG_WAREHOUSE_TYPE: warehouse_type,
        lgmvf.LG_WAREHOUSE_CUSTOM: custom,
        # Baseline
        lgmvf.LG_WAREHOUSE_TOTAL_OPERATING_COSTS: 0,
        lgmvf.LG_WAREHOUSE_ANNUAL_ELECTRICAL_COST: 0,
        lgmvf.LG_WAREHOUSE_ANNUAL_WATER_COST: 0,
        lgmvf.LG_WAREHOUSE_OTHER_OPERATING_COSTS: 0,
        lgmvf.LG_WAREHOUSE_TOTAL_CONSTRUCT_COSTS: 0,
        lgmvf.LG_WAREHOUSE_LAND_COST: 0,
        lgmvf.LG_WAREHOUSE_COST_CONSTRUCT_SQ_M: 0,
        lgmvf.LG_WAREHOUSE_SIZE_SQ_M: 0,
        lgmvf.LG_WAREHOUSE_LEVEL_AMOUNTS: LG_init_dict_by_level(supply_chain_level_ID, num_supply_chain_levels),
        lgmvf.LG_WAREHOUSE_AVG_OPERATING_COSTS_BY_LEVEL: {
            lgmvf.LG_WAREHOUSE_ANNUAL_ELECTRICAL_COST: LG_init_dict_by_level(
                supply_chain_level_ID, num_supply_chain_levels
            ),
            lgmvf.LG_WAREHOUSE_OTHER_OPERATING_COSTS: LG_init_dict_by_level(
                supply_chain_level_ID, num_supply_chain_levels
            ),
            lgmvf.LG_WAREHOUSE_TOTAL_OPERATING_COSTS: LG_init_dict_by_level(
                supply_chain_level_ID, num_supply_chain_levels
            ),
            lgmvf.LG_WAREHOUSE_ANNUAL_WATER_COST: LG_init_dict_by_level(supply_chain_level_ID, num_supply_chain_levels),
        },
        # Target setting
        lgmvf.LG_WAREHOUSE_AVG_CONSTRUCTION_COSTS_BY_LEVEL: {
            lgmvf.LG_WAREHOUSE_LAND_COST: LG_init_dict_by_level(supply_chain_level_ID, num_supply_chain_levels),
            lgmvf.LG_WAREHOUSE_TOTAL_CONSTRUCT_COSTS: LG_init_dict_by_level(
                supply_chain_level_ID, num_supply_chain_levels
            ),
            lgmvf.LG_WAREHOUSE_COST_CONSTRUCT_SQ_M: LG_init_dict_by_level(
                supply_chain_level_ID, num_supply_chain_levels
            ),
            lgmvf.LG_WAREHOUSE_SIZE_SQ_M: LG_init_dict_by_level(supply_chain_level_ID, num_supply_chain_levels),
        },
        lgmvf.LG_WAREHOUSE_NUM_BUILT_BY_YEAR: [0] * num_years,
        lgmvf.LG_WAREHOUSE_NUM_BUILT_BY_YEAR_BY_LEVEL: LG_init_dict_by_level(
            supply_chain_level_ID,
            num_supply_chain_levels,
            [0] * num_years,
        ),
        lgmvf.LG_WAREHOUSE_NUM_REHAB_BY_YEAR: [0] * num_years,
        lgmvf.LG_WAREHOUSE_NUM_REHAB_BY_YEAR_BY_LEVEL: LG_init_dict_by_level(
            supply_chain_level_ID,
            num_supply_chain_levels,
            [0] * num_years,
        ),
        lgmvf.LG_WAREHOUSE_REHAB_COST: 0,
        lgmvf.LG_WAREHOUSE_REHAB_COSTS_BY_LEVEL: LG_init_dict_by_level(supply_chain_level_ID, num_supply_chain_levels),
        lgmvf.LG_WAREHOUSE_NUM_MOVED: 0,
        lgmvf.LG_WAREHOUSE_NUM_MOVED_BY_LEVEL: LG_init_dict_by_level(supply_chain_level_ID, num_supply_chain_levels, 0),
        lgmvf.LG_WAREHOUSE_TARGET_TRANSFER_SOURCE: None,
        lgmvf.LG_WAREHOUSE_TARGET_TRANSFER_SOURCE_BY_LEVEL: LG_init_dict_by_level(
            supply_chain_level_ID, num_supply_chain_levels, None
        ),
        lgmvf.LG_WAREHOUSE_YEAR_MOVED: None,
        lgmvf.LG_WAREHOUSE_YEAR_MOVED_BY_LEVEL: LG_init_dict_by_level(
            supply_chain_level_ID, num_supply_chain_levels, None
        ),
        # Volume analytics
        lgmvf.LG_WAREHOUSE_NUM_RESUPPLY_VISITS_BY_YEAR: [0] * num_years,
        lgmvf.LG_WAREHOUSE_NUM_RESUPPLY_VISITS_BY_YEAR_BY_LEVEL: LG_init_dict_by_level(
            supply_chain_level_ID,
            num_supply_chain_levels,
            [0] * num_years,
        ),
        lgmvf.LG_WAREHOUSE_AVG_PERC_POP_SERVED: [0] * num_years,
        lgmvf.LG_WAREHOUSE_CAPACITY: 0,
        lgmvf.LG_WAREHOUSE_CAPACITY_BY_LEVEL: LG_init_dict_by_level(
            supply_chain_level_ID,
            num_supply_chain_levels,
            0,
        ),
        lgmvf.LG_WAREHOUSE_NUM_MONTHS_SAFETY_STOCK: 0,
        lgmvf.LG_WAREHOUSE_NUM_MONTHS_SAFETY_STOCK_BY_LEVEL: LG_init_dict_by_level(
            supply_chain_level_ID,
            num_supply_chain_levels,
            0,
        ),
        lgmvf.LG_WAREHOUSE_DISTANCE_TO_SUPPLYING_WAREHOUSE: 0,
        lgmvf.LG_WAREHOUSE_DISTANCE_TO_SUPPLYING_WAREHOUSE_BY_LEVEL: LG_init_dict_by_level(
            supply_chain_level_ID,
            num_supply_chain_levels,
            0,
        ),
    }


def LG_init_dict_by_level(
    supply_chain_level_ID: int, num_supply_chain_levels: int, default_value: np.ndarray | dict | list | int | None = 0
):
    result = {}
    for i in range(supply_chain_level_ID + 1, num_supply_chain_levels + 1):
        if isinstance(default_value, (np.ndarray, list)):
            result[i] = default_value.copy()
        else:
            result[i] = default_value
    return result


def LG_init_dict_by_warehouse_by_level(
    warehouse_records: list, num_supply_chain_levels: int, default_value: np.ndarray | list | int = 0
):
    result = {}
    for warehouse_record in warehouse_records:
        warehouse_ID = warehouse_record.get(lgmvf.LG_WAREHOUSE_ID, None)
        warehouse_supply_chain_level_ID = warehouse_record.get(lgmvf.LG_WAREHOUSE_SUPPLY_CHAIN_LEVEL_ID, None)
        result[warehouse_ID] = LG_init_dict_by_level(
            warehouse_supply_chain_level_ID, num_supply_chain_levels, default_value
        )
    return result


def LG_init_dict_by_warehouse(warehouse_records: list, default_value: np.ndarray | list | int = 0):
    result = {}
    for warehouse_record in warehouse_records:
        warehouse_ID = warehouse_record.get(lgmvf.LG_WAREHOUSE_ID, None)
        if isinstance(default_value, (np.ndarray, list)):
            result[warehouse_ID] = default_value.copy()
        else:
            result[warehouse_ID] = default_value
    return result


def LG_init_dict_by_vehicle(vehicle_records: list, default_value: int = 0):
    result = {}
    for vehicle_record in vehicle_records:
        vehicle_ID = vehicle_record.get(lgmvf.LG_VEHICLE_TYPE_ID)
        result[vehicle_ID] = default_value
    return result


#######################################################################################################
#                                                                                                     #
#                                   Vehicle type records                                              #
#                                                                                                     #
#######################################################################################################


def LG_create_vehicle_type_record(
    ID: str,
    name: str,
    string_const: str,
    num_supply_chain_levels: int,
    num_years: int,
    warehouse_records: list = [],
):
    return {
        lgmvf.LG_VEHICLE_TYPE_ID: ID,
        lgmvf.LG_VEHICLE_TYPE_NAME: name,
        lgmvf.LG_VEHICLE_TYPE_STR_CONST: string_const,
        lgmvf.LG_VEHICLE_TYPE_CUSTOM: False,
        # Baseline
        lgmvf.LG_VEHICLE_TYPE_AVG_AMOUNT_BY_LEVEL_AGGR: LG_init_dict_by_level(0, num_supply_chain_levels),
        lgmvf.LG_VEHICLE_TYPE_AVG_AMOUNT_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels
        ),
        lgmvf.LG_VEHICLE_TYPE_AMOUNT_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(warehouse_records),
        lgmvf.LG_VEHICLE_TYPE_UNIT_COST: 0,
        lgmvf.LG_VEHICLE_TYPE_WORKING_LIFE_YEARS: [0] * num_years,
        lgmvf.LG_VEHICLE_TYPE_FUEL_CONSUMPTION: 0,
        lgmvf.LG_VEHICLE_TYPE_REP_MAINT_COST_PERC_CAP_COST: 0,
        lgmvf.LG_VEHICLE_TYPE_NUM_TRIPS: [0] * num_years,
        lgmvf.LG_VEHICLE_TYPE_AVG_KM_PER_TRIP: [0] * num_years,
        lgmvf.LG_VEHICLE_TYPE_COST_PETROL_PER_VEHIC_YEAR: 0,
        # Target setting
        lgmvf.LG_VEHICLE_TYPE_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR: LG_init_dict_by_level(0, num_supply_chain_levels),
        lgmvf.LG_VEHICLE_TYPE_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels
        ),
        lgmvf.LG_VEHICLE_TYPE_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(warehouse_records),
        lgmvf.LG_VEHICLE_TYPE_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR: LG_init_dict_by_level(
            0, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_VEHICLE_TYPE_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_VEHICLE_TYPE_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(
            warehouse_records, [0] * num_years
        ),
        lgmvf.LG_VEHICLE_TYPE_TARG_EXISTING_PLANS_BY_LEVEL_AGGR: LG_init_dict_by_level(
            0, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_VEHICLE_TYPE_TARG_EXISTING_PLANS_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_VEHICLE_TYPE_TARG_EXISTING_PLANS_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(
            warehouse_records, [0] * num_years
        ),
        # Volume analytics
        lgmvf.LG_VEHICLE_TYPE_PERC_SERVICED_BY_LEVEL_AGGR: LG_init_dict_by_level(
            0, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_VEHICLE_TYPE_PERC_SERVICED_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(
            warehouse_records, [0] * num_years
        ),
        lgmvf.LG_VEHICLE_TYPE_PERC_SERVICED_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_VEHICLE_TYPE_DWELL_TIME_BY_LEVEL_AGGR: LG_init_dict_by_level(0, num_supply_chain_levels),
        lgmvf.LG_VEHICLE_TYPE_DWELL_TIME_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(warehouse_records),
        lgmvf.LG_VEHICLE_TYPE_DWELL_TIME_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels
        ),
        lgmvf.LG_VEHICLE_TYPE_CAPACITY: 0,
        lgmvf.LG_VEHICLE_TYPE_AVG_SPEED: 0,
        lgmvf.LG_VEHICLE_TYPE_MAX_DAYS_ON_ROUTE: 0,
    }


def LG_init_vehicle_type_records(num_supply_chain_levels: int, num_years: int):
    vehicle_type_records = []

    for default_vehicle_type in lgc.LG_DEFAULT_VEHICLE_TYPES:
        default_vehicle_rec = LG_create_vehicle_type_record(
            str(default_vehicle_type[lgmvf.LG_VEHICLE_TYPE_ID]),
            default_vehicle_type[lgmvf.LG_VEHICLE_TYPE_NAME],
            default_vehicle_type[lgmvf.LG_VEHICLE_TYPE_STR_CONST],
            num_supply_chain_levels,
            num_years,
            [],
        )
        vehicle_type_records.append(default_vehicle_rec)

    return vehicle_type_records


#######################################################################################################
#                                                                                                     #
#                                   Worker type records                                               #
#                                                                                                     #
#######################################################################################################


def LG_create_worker_type_record(
    ID: str,
    name: str,
    string_const: str,
    num_supply_chain_levels: int,
    num_years: int,
    vehicle_records: list = [],
    warehouse_records: list = [],
):
    return {
        lgmvf.LG_WORKER_TYPE_ID: ID,
        lgmvf.LG_WORKER_TYPE_NAME: name,
        lgmvf.LG_WORKER_TYPE_STR_CONST: string_const,
        lgmvf.LG_WORKER_TYPE_CUSTOM: False,
        # Baseline
        lgmvf.LG_WORKER_TYPE_ANNUAL_WAGES: 0.0,
        lgmvf.LG_WORKER_TYPE_ANNUAL_REAL_SALARY_INC: 0,
        lgmvf.LG_WORKER_TYPE_ANNUAL_BENEFITS: 0.0,
        lgmvf.LG_WORKER_TYPE_ANNUAL_BENEFITS_ADJUST: 0,
        lgmvf.LG_WORKER_TYPE_AVG_AMOUNT_BY_LEVEL_AGGR: LG_init_dict_by_level(0, num_supply_chain_levels),
        lgmvf.LG_WORKER_TYPE_AVG_AMOUNT_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels
        ),
        lgmvf.LG_WORKER_TYPE_AMOUNT_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(warehouse_records),
        # Target setting
        lgmvf.LG_WORKER_TYPE_TARG_CONFIG_OPTION: lgc.LG_TARGET_CONFIG_EXISTING_PLANS_MST_ID,
        lgmvf.LG_WORKER_TYPE_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR: LG_init_dict_by_level(0, num_supply_chain_levels),
        lgmvf.LG_WORKER_TYPE_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels
        ),
        lgmvf.LG_WORKER_TYPE_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(warehouse_records),
        lgmvf.LG_WORKER_TYPE_NUM_EXISTING_PLANS_BY_LEVEL_AGGR: LG_init_dict_by_level(
            0, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_WORKER_TYPE_NUM_EXISTING_PLANS_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_WORKER_TYPE_NUM_EXISTING_PLANS_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(
            warehouse_records, [0] * num_years
        ),
        lgmvf.LG_WORKER_TYPE_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR: LG_init_dict_by_level(
            0, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_WORKER_TYPE_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_WORKER_TYPE_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(
            warehouse_records, [0] * num_years
        ),
        lgmvf.LG_WORKER_TYPE_VEHICLE_STD: LG_init_dict_by_vehicle(vehicle_records, 1),
        # Volume analytics
        lgmvf.LG_WORKER_TYPE_CLASSIFICATION: lgc.LG_WORKER_CLASSIFICATION_OTHER,
        lgmvf.LG_WORKER_TYPE_CAPACITY: 0,
    }


def LG_init_worker_type_records(
    num_supply_chain_levels: int,
    vehicle_type_records: list,
    num_years: int,
):
    worker_type_records = []

    for default_worker_type in lgc.LG_DEFAULT_WORKER_TYPES:
        default_worker_rec = LG_create_worker_type_record(
            str(default_worker_type[lgmvf.LG_WORKER_TYPE_ID]),
            default_worker_type[lgmvf.LG_WORKER_TYPE_NAME],
            default_worker_type[lgmvf.LG_WORKER_TYPE_STR_CONST],
            num_supply_chain_levels,
            num_years,
            vehicle_type_records,
            [],
        )
        worker_type_records.append(default_worker_rec)

    return worker_type_records


#######################################################################################################
#                                                                                                     #
#                                   Cold chain records                                                #
#                                                                                                     #
#######################################################################################################


def LG_create_cold_chain_record(
    ID: str,
    name: str,
    string_const: str,
    cold_chain_type: int,
    num_supply_chain_levels: int,
    num_years: int,
    warehouse_records: list = [],
):
    return {
        lgmvf.LG_COLD_CHAIN_ID: ID,
        lgmvf.LG_COLD_CHAIN_NAME: name,
        lgmvf.LG_COLD_CHAIN_STR_CONST: string_const,
        lgmvf.LG_COLD_CHAIN_CUSTOM: False,
        lgmvf.LG_COLD_CHAIN_TYPE: cold_chain_type,
        # Baseline
        lgmvf.LG_COLD_CHAIN_AVG_AMOUNT_BY_LEVEL_AGGR: LG_init_dict_by_level(0, num_supply_chain_levels),
        lgmvf.LG_COLD_CHAIN_AVG_AMOUNT_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels
        ),
        lgmvf.LG_COLD_CHAIN_AMOUNT_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(warehouse_records),
        lgmvf.LG_COLD_CHAIN_UNIT_COST: 0,
        lgmvf.LG_COLD_CHAIN_CAPACITY: 0,
        lgmvf.LG_COLD_CHAIN_MAINT_COST_PERC_CAP_COST: 0,
        lgmvf.LG_COLD_CHAIN_WORKING_LIFE_YEARS: [0] * num_years,
        # Target setting
        lgmvf.LG_COLD_CHAIN_TARG_WAREHOUSE_STD_BY_LEVEL_AGGR: LG_init_dict_by_level(0, num_supply_chain_levels),
        lgmvf.LG_COLD_CHAIN_TARG_WAREHOUSE_STD_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels
        ),
        lgmvf.LG_COLD_CHAIN_TARG_WAREHOUSE_STD_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(warehouse_records),
        lgmvf.LG_COLD_CHAIN_TARG_NUM_AVAILABLE_BY_LEVEL_AGGR: LG_init_dict_by_level(
            0, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_COLD_CHAIN_TARG_NUM_AVAILABLE_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_COLD_CHAIN_TARG_NUM_AVAILABLE_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(
            warehouse_records, [0] * num_years
        ),
        lgmvf.LG_COLD_CHAIN_TARG_EXISTING_PLANS_BY_LEVEL_AGGR: LG_init_dict_by_level(
            0, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_COLD_CHAIN_TARG_EXISTING_PLANS_BY_LEVEL_DISAG: LG_init_dict_by_warehouse_by_level(
            warehouse_records, num_supply_chain_levels, [0] * num_years
        ),
        lgmvf.LG_COLD_CHAIN_TARG_EXISTING_PLANS_BY_WAREHOUSE_DISAG: LG_init_dict_by_warehouse(
            warehouse_records, [0] * num_years
        ),
    }


def LG_init_cold_chain_records(num_supply_chain_levels: int, num_years: int):
    cold_chain_records = []

    for default_cold_chain_item in lgc.LG_DEFAULT_COLD_CHAIN_RECORDS:
        default_cold_chain_rec = LG_create_cold_chain_record(
            str(default_cold_chain_item[lgmvf.LG_COLD_CHAIN_ID]),
            default_cold_chain_item[lgmvf.LG_COLD_CHAIN_NAME],
            default_cold_chain_item[lgmvf.LG_COLD_CHAIN_STR_CONST],
            default_cold_chain_item[lgmvf.LG_COLD_CHAIN_TYPE],
            num_supply_chain_levels,
            num_years,
            [],
        )
        lgmvu.set_LG_cold_chain_rec_capacity(
            default_cold_chain_rec, default_cold_chain_item[lgmvf.LG_COLD_CHAIN_CAPACITY]
        )
        lgmvu.set_LG_cold_chain_rec_unit_cost(
            default_cold_chain_rec, default_cold_chain_item[lgmvf.LG_COLD_CHAIN_UNIT_COST]
        )
        cold_chain_records.append(default_cold_chain_rec)

    return cold_chain_records


def LG_init_default_cold_chain_records(default_cold_chain_records_DB):
    default_cold_chain_records = []

    for cold_chain_DB_obj in default_cold_chain_records_DB:
        default_cold_chain_records.append(
            {
                lgmvf.LG_COLD_CHAIN_ID: str(cold_chain_DB_obj[lgdbk.LG_COLD_CHAIN_ID_KEY]),
                lgmvf.LG_COLD_CHAIN_TYPE: cold_chain_DB_obj[lgdbk.LG_COLD_CHAIN_TYPE_KEY],
                lgmvf.LG_COLD_CHAIN_NAME: cold_chain_DB_obj[lgdbk.LG_COLD_CHAIN_NAME_KEY],
            }
        )

    return default_cold_chain_records


#######################################################################################################
#                                                                                                     #
#                                   Third party logistics type records                                #
#                                                                                                     #
#######################################################################################################


def LG_create_third_party_logistics_type_record(ID: str, name: str, string_const: str, num_years: int):
    return {
        lgmvf.LG_TPL_CONTRACT_ID: ID,
        lgmvf.LG_TPL_CONTRACT_NAME: name,
        lgmvf.LG_TPL_CONTRACT_STR_CONST: string_const,
        lgmvf.LG_TPL_CONTRACT_CUSTOM: False,
        lgmvf.LG_TPL_CONTRACT_COST: [0] * num_years,
    }


#######################################################################################################
#                                                                                                     #
#                                            Results                                                  #
#                                                                                                     #
#######################################################################################################


def LG_init_result_dict_by_level(modvars):
    supply_chain_level_records = modvars[lgmvt.LG_SUPPLY_CHAIN_LEVEL_RECORDS_TAG]
    num_supply_chain_levels = lgmvu.get_LG_num_supply_chain_levels(supply_chain_level_records)
    first_year_mv = modvars[pjmvt.PJN_FirstYearTag]
    final_year_mv = modvars[pjmvt.PJN_FinalYearTag]
    num_years = final_year_mv - first_year_mv + 1

    return LG_init_dict_by_level(0, num_supply_chain_levels, np.full(num_years, 0.0))


def LG_init_result_dict_by_warehouse(modvars):
    disag_level = modvars[lgmvt.LG_DISAGGREGATION_LEVEL_TAG]
    warehouse_records = modvars[lgmvt.LG_WAREHOUSE_RECORDS_TAG]
    first_year_mv = modvars[pjmvt.PJN_FirstYearTag]
    final_year_mv = modvars[pjmvt.PJN_FinalYearTag]

    warehouse_records_to_include = lgmvu.get_LG_warehouse_records_up_to_disag_level(warehouse_records, disag_level)
    num_years = final_year_mv - first_year_mv + 1

    return LG_init_dict_by_warehouse(warehouse_records_to_include, np.full(num_years, 0.0))


def LG_init_result_dict_by_warehouse_by_level(modvars):
    disag_level = modvars[lgmvt.LG_DISAGGREGATION_LEVEL_TAG]
    supply_chain_level_records = modvars[lgmvt.LG_SUPPLY_CHAIN_LEVEL_RECORDS_TAG]
    warehouse_records = modvars[lgmvt.LG_WAREHOUSE_RECORDS_TAG]
    first_year_mv = modvars[pjmvt.PJN_FirstYearTag]
    final_year_mv = modvars[pjmvt.PJN_FinalYearTag]

    num_supply_chain_levels = lgmvu.get_LG_num_supply_chain_levels(supply_chain_level_records)
    warehouse_records_to_include = lgmvu.get_LG_warehouse_records_for_disag_level(warehouse_records, disag_level)
    num_years = final_year_mv - first_year_mv + 1

    return LG_init_dict_by_warehouse_by_level(
        warehouse_records_to_include, num_supply_chain_levels, np.full(num_years, 0.0)
    )


def LG_init_result_dict_by_vehicle_by_level(modvars):
    vehicle_records = modvars[lgmvt.LG_VEHICLE_TYPE_RECORDS_TAG]
    return {
        vehicle_record.get(lgmvf.LG_VEHICLE_TYPE_ID): LG_init_result_dict_by_level(modvars)
        for vehicle_record in vehicle_records
    }


def LG_init_result_dict_by_vehicle_by_warehouse(modvars):
    vehicle_records = modvars[lgmvt.LG_VEHICLE_TYPE_RECORDS_TAG]
    return {
        vehicle_record.get(lgmvf.LG_VEHICLE_TYPE_ID): LG_init_result_dict_by_warehouse(modvars)
        for vehicle_record in vehicle_records
    }


def LG_init_result_dict_by_vehicle_by_warehouse_by_level(modvars):
    vehicle_records = modvars[lgmvt.LG_VEHICLE_TYPE_RECORDS_TAG]
    return {
        vehicle_record.get(lgmvf.LG_VEHICLE_TYPE_ID): LG_init_result_dict_by_warehouse_by_level(modvars)
        for vehicle_record in vehicle_records
    }


def LG_init_result_dict_by_worker_type_by_level(modvars):
    worker_records = modvars[lgmvt.LG_WORKER_TYPE_RECORDS_TAG]
    return {
        worker_record.get(lgmvf.LG_WORKER_TYPE_ID): LG_init_result_dict_by_level(modvars)
        for worker_record in worker_records
    }


def LG_init_result_dict_by_worker_type_by_warehouse(modvars):
    worker_records = modvars[lgmvt.LG_WORKER_TYPE_RECORDS_TAG]
    return {
        worker_record.get(lgmvf.LG_WORKER_TYPE_ID): LG_init_result_dict_by_warehouse(modvars)
        for worker_record in worker_records
    }


def LG_init_result_dict_by_worker_type_by_warehouse_by_level(modvars):
    worker_records = modvars[lgmvt.LG_WORKER_TYPE_RECORDS_TAG]
    return {
        worker_record.get(lgmvf.LG_WORKER_TYPE_ID): LG_init_result_dict_by_warehouse_by_level(modvars)
        for worker_record in worker_records
    }


def LG_init_result_dict_by_cold_chain_by_level(modvars):
    cold_chain_records = modvars[lgmvt.LG_COLD_CHAIN_RECORDS_TAG]
    return {
        cold_chain_record.get(lgmvf.LG_COLD_CHAIN_ID): LG_init_result_dict_by_level(modvars)
        for cold_chain_record in cold_chain_records
    }


def LG_init_result_dict_by_cold_chain_by_warehouse(modvars):
    cold_chain_records = modvars[lgmvt.LG_COLD_CHAIN_RECORDS_TAG]
    return {
        cold_chain_record.get(lgmvf.LG_COLD_CHAIN_ID): LG_init_result_dict_by_warehouse(modvars)
        for cold_chain_record in cold_chain_records
    }


def LG_init_result_dict_by_cold_chain_by_warehouse_by_level(modvars):
    cold_chain_records = modvars[lgmvt.LG_COLD_CHAIN_RECORDS_TAG]
    return {
        cold_chain_record.get(lgmvf.LG_COLD_CHAIN_ID): LG_init_result_dict_by_warehouse_by_level(modvars)
        for cold_chain_record in cold_chain_records
    }


def LG_init_quantity_source_drug_supply_year_arr(num_drugs_supplies: int, num_years: int, value=0.0):
    return np.full((lgc.LG_NUM_QUANTITY_SOURCES, num_drugs_supplies, num_years), value, gbc.GB_CALC_FLOAT_TYPE)



from SpectrumCommon.Const.IH import *
import SpectrumCommon.Const.IH.IHTags as ihmvt
from SpectrumCommon.Modvars.GB.GBDefs import *
# from SpectrumCommon.Modvars.IH.IHIndices import *


ih_program_area_types_dim = 'program_area_types'
ih_program_area_types_ids = {
    IH_MATERNAL_NEWBORN_HEALTH_MST_ID : 'IH_MATERNAL_NEWBORN_HEALTH_MST_ID'   ,
    IH_SEXUAL_REPROD_HEALTH_MST_ID    : 'IH_SEXUAL_REPROD_HEALTH_MST_ID',
    UH_ChildHealthMstID               : 'UH_ChildHealthMstID',
    UH_VaccinationMstID               : 'UH_VaccinationMstID',
    UH_MalariaMstID                   : 'UH_MalariaMstID',
    IH_TB_MST_ID                      : 'IH_TB_MST_ID',
    IH_HIV_AIDS_MST_ID                : 'IH_HIV_AIDS_MST_ID',
    UH_NutritionMstID                 : 'UH_NutritionMstID',
    UH_WASHMstID                      : 'UH_WASHMstID',
    IH_NCD_MST_ID                     : 'IH_NCD_MST_ID',
    UH_MentalNueroSubsUseDisMstID     : 'UH_MentalNueroSubsUseDisMstID',
    UH_AdolescentHealthMstID          : 'UH_AdolescentHealthMstID',
    IH_NTD_MST_ID                     : 'IH_NTD_MST_ID'
}

ih_subgroup_types_dim = 'subgroup_types'
ih_subgroup_types_ids = {
    5: "Management of ectopic pregnancy care",
    6: "Pregancy care - ANC",
    7: "Pregnancy care - Treatment of pregnancy complications",
    8: "Childbirth care - Facility births",
    9: "Childbirth care - Home births",
    10: "Childbirth care - Other",
    11: "Postpartum care - Treatment of sepsis",
    12: "Postpartum care - Treatment of newborn sepsis",
    13: "Postpartum care - Other",
    14: "Other sexual and reproductive health",
    42: "Other",
    112: "Family planning",
    113: "Abortion",
    85: "General",
    16: "Diarrhea management",
    17: "Pneumonia",
    18: "Malaria",
    19: "Measles",
    86: "General",
    79: "Prevention",
    80: "Case management",
    90: "Screening (provider-initiated)",
    91: "TB infection prevention, testing (provider-initiated)",
    92: "TB infection prevention, medication (provider-initiated)",
    93: "Case finding (patient-initiated)",
    94: "Diagnosis (patient-initiated)",
    95: "Diagnosis (provider-initiated)",
    96: "Drug resistance detection",
    97: "Treatment, drug-susceptible",
    98: "Treatment, drug-resistant",
    99: "Surgery",
    100: "First-line drug treatment monitoring ",
    101: "Second-line drug treatment monitoring ",
    102: "Adverse events treatment",
    103: "Non-medical support",
    104: "Palliative care",
    108: "TB and other diseases infection prevention, medication",
    109: "HIV treatment, first-line",
    110: "Vaccination",
    105: "Prevention",
    106: "Testing, treatment and care",
    107: "Social enablers",
    53: "Women of reproductive age and adolescent girls",
    54: "Pregnant and lactating women",
    55: "Adults",
    56: "All populations",
    57: "Children",
    87: "General",
    34: "CVD & diabetes",
    35: "Breast cancer",
    36: "Cervical cancer",
    72: "Colorectal cancer",
    39: "Oral care and cancer",
    40: "Respiratory disease",
    41: "Emergency care",
    70: "Risk Factors",
    71: "Policy interventions - Under Programme Costing",
    68: "Anxiety disorders",
    43: "Depression",
    44: "Psychosis",
    45: "Bipolar disorder",
    46: "Epilepsy",
    47: "Developmental disorders",
    48: "Conduct disorders",
    67: "Attention disorders",
    49: "Dementia",
    50: "Alcohol use/dependence",
    51: "Drug use/dependence",
    52: "Self-harm/suicide",
    88: "General",
    75: "Mass drug administration",
    76: "Disease management",
    77: "Vector control",
    78: "Veterinary public health"
}


ih_deliv_chan_types_dim = 'delivery_channel_types'
ih_deliv_chan_types_ids = {
    1: "Community",
    2: "Outreach",
    5: "Prehospital emergency",
    6: "General outpatient services",
    4: "First referral",
    7: "Second referral and above",
}

ih_health_sys_admin_unit_types_dim = 'health_sys_admin_unit_types'
ih_health_sys_admin_unit_types_ids = {
    1: "National",
    2: "Province",
    3: "District",
    4: "Admin level 4",
    5: "Admin level 5",
}

ih_staff_types_dim = 'staff_types'
ih_staff_types_ids = {
    HW_GENERAL_MEDICAL_PRACTITIONERS_CURR_ID           : 'General medical practitioners',
    HW_SPECIAL_MEDICAL_PRACTITIONERS_CURR_ID           : 'Special medical practitioners',
    HW_CLINIC_OFF_SURG_TECHS_CURR_ID                   : 'Clinic off surg techs',
    HW_NURSING_PROFESSIONALS_CURR_ID                   : 'Nursing professionals',
    HW_MIDWIFERY_PROFESSIONALS_CURR_ID                 : 'Midwifery professionals',
    HW_MIDWIFERY_ASSOCIATE_PROFESSIONALS_CURR_ID       : 'Midwifery associate professionals',
    HW_NURSING_ASSOCIATE_PROFESSIONALS_CURR_ID         : 'Nursing associate professionals',
    HW_MED_PATH_LAB_TECHS_CURR_ID                      : 'Med path lab techs',
    HW_PHARM_TECHS_ASSIST_CURR_ID                      : 'Pharm techs assist',
    HW_MED_IMAGING_THERA_EQUIP_TECHS_CURR_ID           : 'Med imaging thera equip techs',
    HW_AMBULANCE_WORKERS_CURR_ID                       : 'Ambulance workers',
    HW_COMM_HLTH_WORKERS_CURR_ID                       : 'Comm hlth workers',
    HW_OTHER_STAFF_TYPE_CURR_ID                        : 'Other staff type',
    HW_DENTISTS_CURR_ID                                : 'Dentists',
    HW_DENTAL_ASSIST_THERAPISTS_CURR_ID                : 'Dental assist therapists',
    HW_MED_DENTAL_PROSTHETIC_REL_TECHS_CURR_ID         : 'Med dental prosthetic rel techs',
    HW_PHARMACISTS_CURR_ID                             : 'Pharmacists',
    HW_PARAMED_PRACTITIONERS_CURR_ID                   : 'Paramed practitioners',
    HW_ENVIRON_OCCUPAT_HEALTH_HYGIENE_PROF_CURR_ID     : 'Environ occupat health hygiene prof',
    HW_DIETICIANS_NUTRITIONISTS_CURR_ID                : 'Dieticians nutritionists',
    HW_AUDIOLOGISTS_SPEECH_THERAPISTS_CURR_ID          : 'Audiologists speech therapists',
    HW_OPTOMETRISTS_OPHTHALMIC_OPTICIANS_CURR_ID       : 'Optometrists ophthalmic opticians',
    HW_DISPENSING_OPTICIANS_CURR_ID                    : 'Dispensing opticians',
    HW_ENVIRON_OCCUPAT_HEALTH_INSPEC_ASSO_CURR_IDC     : 'Environ occupat health inspec asso curr idc',
    HW_TRAD_COMP_MED_PROF_CURR_ID                      : 'Trad comp med prof',
    HW_TRAD_COMP_MED_ASSOC_PROF_CURR_ID                : 'Trad comp med assoc prof',
    HW_HEALTH_CARE_ASSISTS_CURR_ID                     : 'Health care assists',
    HW_HOME_BASED_PERS_CARE_WORKERS_CURR_ID            : 'Home based pers care workers',
    HW_PERS_CARE_WORKERS_HEALTH_SERV_NOT_CLASS_CURR_ID : 'Pers care workers health serv not class',
    HW_PHYSIOTHERAPISTS_CURR_ID                        : 'Physiotherapists',
    HW_PHYSIOTHERAPY_TECHS_ASSISTS_CURR_ID             : 'Physiotherapy techs assists',
    HW_HEALTH_ASSOC_PROF_NOT_CLASS_CURR_ID             : 'Health assoc prof not class',
    HW_PSYCHOLOGISTS_CURR_ID                           : 'Psychologists',
    HW_SOCIAL_WORK_COUNSELLING_PROF_CURR_ID            : 'Social work counselling prof',
    HW_SOCIAL_WORK_ASSOC_PROF_CURR_ID                  : 'Social work assoc prof',
}
# tb_patient_shape = lambda d, **kwargs : gb_shape(d,'1D float',
#                                                   dim=['tb_patient'], 
#                                                   indices=[tb_patient_indices], **kwargs)


IH_modvar_shapes = {
    
        ihmvt.IH_VERSION_TAG_V1                                    : gb_int_shape('IHT version, may be deprecated'),
        ihmvt.IH_HLTH_SERV_ANAL_MODE_MST_ID_TAG_V1                 : gb_int_shape('Health serv analyses mode master id'),  # Programme or Delivery Channel Mode
        ihmvt.IH_PROG_AREA_DELIV_CHAN_FILTER_MST_ID_TAG_V1         : gb_int_shape('Program area/delivery channel filter master id'), # programme area or delivery channel mstID
        ihmvt.IH_MAX_CUST_INTERV_MST_ID_TAG_V1                     : gb_int_shape('Maximum custom intervention master id'),      
        ihmvt.IH_MOH_BUDGET_TAG_V1                                 : gb_float_shape('MOH budget'),                 
        ihmvt.IH_DATA_CHANGED_TAG_V1                               : gb_bool_shape('Data changed, may be deprecated'), 
        ihmvt.IH_PROJ_VALID_TAG_V1                                 : gb_bool_shape('Projection valid, may be deprecated'), 
        ihmvt.IH_USING_ADOL_STRATA_MODE_TAG_V1                     : gb_bool_shape('Use adol strata flag'),
        ihmvt.IH_USING_DIS_SPEC_COST_MODE_TAG_V1                   : gb_bool_shape('Use Dis Spec Codst flag'), 
        ihmvt.IH_USING_PHC_MODE_TAG_V1                             : gb_bool_shape('Use PHC flag'), 
        ihmvt.IH_USING_DEF_MED_STAFF_TYPES_TAG_V1                  : gb_bool_shape('Use default medical staff types flag'), 
        ihmvt.IH_USING_DEF_FAC_TYPES_TAG_V1                        : gb_bool_shape('Use default facility types flag'), 
        ihmvt.IH_USING_MOH_BUDGET_RES_TAG_V1                       : gb_bool_shape('Use MOH budget res flag flag'), 
        ihmvt.IH_USING_HPV_PROG_AREA_TAG_V1                        : gb_bool_shape('Use HPV program areas flag flag'), 
        ihmvt.IH_USING_WORK_HLTH_PROG_AREA_TAG_V1                  : gb_bool_shape('Use work health program area flag'), 
        ihmvt.IH_USING_TP_FROM_TIME_TAG_V1                         : gb_bool_shape('Use TB from time flag'), 
        ihmvt.IH_USING_TB_IMPACT_CALCS_TP_TAG_V1                   : gb_bool_shape('Use TB impact calcs flag'), 
        ihmvt.IH_USING_TB_DAT_TP_TAG_V1                            : gb_bool_shape('Use TB data TP flag'), 
        ihmvt.IH_USING_LOCAL_CURR_TAG_V1                           : gb_bool_shape('Use local currrency flag'), 
        ihmvt.IH_FILE_NAME_TAG_V1                                  : gb_str_shape('File name, may be deprecated'),                         
        ihmvt.IH_PROJ_VALID_DATE_TAG_V1                            : gb_str_shape('projection date, may be deprecated'),           
        ihmvt.IH_PROG_AREA_RECORDS_TAG_V1                          : gb_shape('Program area records', type='1D dict', 
                                                                            dim=[ih_program_area_types_dim],
                                                                            indices=[ih_program_area_types_ids], 
                                                                            innerShape={
                                                                                'progAreaID'  : gb_shape('Program area ID', type='str'),
                                                                                'name'  : gb_shape('Program area name', type='str'),
                                                                                'stringConstant'  : gb_shape('string constant', type='str'),
                                                                                'subgroupRecords'  : gb_shape('subgroup records', type='1D dict', 
                                                                                                            dim=[ih_subgroup_types_dim],
                                                                                                            indices=[ih_subgroup_types_ids], 
                                                                                                            innerShape={
                                                                                                                'subgroupID'  : gb_shape('Subgroup ID', type='str'),
                                                                                                                'name'  : gb_shape('Subgroup name', type='str'),
                                                                                                                'stringConstant'  : gb_shape('string constant', type='str'),
                                                                                                                'active'  : gb_bool_shape('active flag'),
                                                                                                                'custom'  : gb_bool_shape('Custom area flag'),
                                                                                                                }),
                                                                                'active'  : gb_bool_shape('active flag'),
                                                                                'custom'  : gb_bool_shape('Custom area flag'),
                                                                            },
                                                                            has_default=False
                                                                        ),       
        ihmvt.IH_DELIV_CHAN_RECORDS_TAG_V1                         : gb_shape('Delivery channel records', type='1D dict', 
                                                                            dim=[ih_deliv_chan_types_dim],
                                                                            indices=[ih_deliv_chan_types_ids],
                                                                            innerShape={
                                                                                'delivChanID'  : gb_shape('Delivery channel ID', type='str'),
                                                                                'name'  : gb_shape('Delivery channel name', type='str'),
                                                                                'stringConstant'  : gb_shape('string constant', type='str'),
                                                                                'active'  : gb_bool_shape('active flag'),
                                                                                'custom'  : gb_bool_shape('Custom area flag'),
                                                                            },
                                                                            has_default=False
                                                                        ),       
                                                                                        
        ihmvt.IH_PERC_PROG_MGMT_COSTS_PHC_VIS_TAG_V1               : gb_bool_shape('Percentage program management costs per PHC visit flag'),       
        ihmvt.IH_HR_FACTOR_PHC_TAG_V1                              : gb_float_shape('Health resource factor per PHC visit'),         
        ihmvt.IH_LG_PERC_DRUG_SUPP_COSTS_PHC_TAG_V1                : gb_int_shape('Logistics percentage drug/supply costs per PHC visit'),    
        ihmvt.IH_SHOW_INACTIVE_PROG_AREAS_EXIST_INTERV_V1          : gb_bool_shape('Show inactive program areas exist interventions flag'),
        ihmvt.IH_SHOW_INACTIVE_PROG_AREAS_EXIST_DRUG_SUPP_V1       : gb_bool_shape('Show inactive program areas exist drugs/supply flag'),
        ihmvt.IH_ENTER_UNIT_COSTS_ALL_YEARS_DRUG_SUPP_V1           : gb_bool_shape('Enter unit costs all years drug/supply flag'),
        ihmvt.IH_HEALTH_SYS_ADMIN_UNIT_RECORDS_TAG                 : gb_shape('Health system records', type='1D dict', 
                                                                            dim=[ih_health_sys_admin_unit_types_dim],
                                                                            indices=[ih_health_sys_admin_unit_types_ids],
                                                                            innerShape={
                                                                                'healthSysAdminUnitID'  : gb_shape('Health system ID', type='str'),
                                                                                'name'  : gb_shape('Health system name', type='str'),
                                                                                'stringConstant'  : gb_shape('string constant', type='str'),
                                                                                'active'  : gb_bool_shape('active flag'),
                                                                                'custom'  : gb_bool_shape('Custom area flag'),
                                                                                'number'  : gb_int_shape('Health system number'),
                                                                            },
                                                                            has_default=False
                                                                        ), 

        # Results
        ihmvt.IH_RES_STAFF_CAP_UTILIZATION_STATIC_TAG_V1           : gb_shape('Staff capacity utilization static results', '3D float',
                                                                    dim=[ih_program_area_types_dim, ih_staff_types_dim, gb_year_dim], 
                                                                    indices=[ih_program_area_types_ids, ih_staff_types_ids, gb_year_indices]),
        ihmvt.IH_RES_STAFF_CAP_UTILIZATION_DYNAMIC_TAG_V1          : gb_shape('Staff capacity utilization dynamic results', '3D float',
                                                                    dim=[ih_program_area_types_dim, ih_staff_types_dim, gb_year_dim], 
                                                                    indices=[ih_program_area_types_ids, ih_staff_types_ids, gb_year_indices]),
}